@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2021 khal contributors
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,15 +20,23 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
+import datetime as dt
 import glob
 import logging
 import os
+import pathlib
 from os.path import expanduser, expandvars, join
+from typing import Callable, Iterable, Literal, Optional, Union
 
 import pytz
 import xdg
 from tzlocal import get_localzone
-from validate import VdtValueError
+
+try:
+    # Available from configobj 5.1.0
+    from configobj.validate import VdtValueError
+except ModuleNotFoundError:
+    from validate import VdtValueError
 
 from ..khalendar.vdir import CollectionNotFoundError, Vdir
 from ..parse_datetime import guesstimedeltafstr
@@ -38,33 +46,35 @@ from .exceptions import InvalidSettingsError
 logger = logging.getLogger('khal')
 
 
-def is_timezone(tzstring):
-    """tries to convert tzstring into a pytz timezone
+def is_timezone(tzstring: Optional[str]) -> dt.tzinfo:
+    """tries to convert tzstring into a pytz timezone or return local timezone
 
     raises a VdtvalueError if tzstring is not valid
     """
     if not tzstring:
-        return get_localzone()
+        # later version of tzlocal return zoneinfo (not pytz) timezones
+        # as a lot of our other code can't deal with this yet, we need to force
+        # pytz timezones for the time being
+        return pytz.timezone(str(get_localzone()))
     try:
         return pytz.timezone(tzstring)
     except pytz.UnknownTimeZoneError:
         raise VdtValueError(f"Unknown timezone {tzstring}")
 
 
-def is_timedelta(string):
+def is_timedelta(string: str) -> dt.timedelta:
     try:
         return guesstimedeltafstr(string)
     except ValueError:
         raise VdtValueError(f"Invalid timedelta: {string}")
 
 
-def weeknumber_option(option):
+def weeknumber_option(option: str) -> Union[str, Literal[False]]:
     """checks if *option* is a valid value
 
     :param option: the option the user set in the config file
     :type option: str
-    :returns: off, left, right
-    :rtype: str/bool
+    :returns: 'off', 'left', 'right' or False
     """
     option = option.lower()
     if option == 'left':
@@ -79,13 +89,11 @@ def weeknumber_option(option):
             "'off', 'left' or 'right'")
 
 
-def monthdisplay_option(option):
+def monthdisplay_option(option: str) -> str:
     """checks if *option* is a valid value
 
     :param option: the option the user set in the config file
-    :type option: str
     :returns: firstday, firstfullweek
-    :rtype: str/bool
     """
     option = option.lower()
     if option == 'firstday':
@@ -99,19 +107,19 @@ def monthdisplay_option(option):
         )
 
 
-def expand_path(path):
+def expand_path(path: str) -> str:
     """expands `~` as well as variable names"""
     return expanduser(expandvars(path))
 
 
-def expand_db_path(path):
+def expand_db_path(path: str) -> str:
     """expands `~` as well as variable names, defaults to $XDG_DATA_HOME"""
     if path is None:
         path = join(xdg.BaseDirectory.xdg_data_home, 'khal', 'khal.db')
     return expanduser(expandvars(path))
 
 
-def is_color(color):
+def is_color(color: str) -> str:
     """checks if color represents a valid color
 
     raises a VdtValueError if color is not valid
@@ -131,7 +139,7 @@ def is_color(color):
     raise VdtValueError(color)
 
 
-def test_default_calendar(config):
+def test_default_calendar(config) -> None:
     """test if config['default']['default_calendar'] is set to a sensible
     value
     """
@@ -149,7 +157,7 @@ def test_default_calendar(config):
         raise InvalidSettingsError()
 
 
-def get_color_from_vdir(path):
+def get_color_from_vdir(path: str) -> Optional[str]:
     try:
         color = Vdir(path, '.ics').get_meta('color')
     except CollectionNotFoundError:
@@ -166,7 +174,7 @@ def get_color_from_vdir(path):
     return color
 
 
-def get_unique_name(path, names):
+def get_unique_name(path: str, names: Iterable[str]) -> str:
     # TODO take care of edge cases, make unique name finding less brain-dead
     try:
         name = Vdir(path, '.ics').get_meta('displayname')
@@ -182,23 +190,39 @@ def get_unique_name(path, names):
     return name
 
 
-def get_all_vdirs(path):
+def get_all_vdirs(expand_path: str) -> Iterable[str]:
     """returns a list of paths, expanded using glob
     """
-    items = glob.glob(path)
-    return items
+    # FIXME currently returns a list of all directories in path
+    # we add an additional / at the end to make sure we are only getting
+    # directories
+    items = glob.glob(f'{expand_path}/', recursive=True)
+    paths = [pathlib.Path(item) for item in sorted(items, key=len, reverse=True)]
+    leaves = set()
+    parents = set()
+    for path in paths:
+        if path in parents:
+            # we have already seen the current directory as the parent of
+            # another directory, so this directory can't be a vdir
+            continue
+        parents.add(path.parent)
+        leaves.add(path)
+    # sort to make sure that auto generated names are always identical
+    return sorted(os.fspath(path) for path in leaves)
 
 
-def get_vdir_type(_):
+def get_vdir_type(_: str) -> str:
     # TODO implement
     return 'calendar'
 
 
 def config_checks(
-        config,
-        _get_color_from_vdir=get_color_from_vdir,
-        _get_vdir_type=get_vdir_type):
+    config,
+    _get_color_from_vdir: Callable=get_color_from_vdir,
+    _get_vdir_type: Callable=get_vdir_type,
+) -> None:
     """do some tests on the config we cannot do with configobj's validator"""
+    # TODO rename or split up, we are also expanding vdirs of type discover
     if len(config['calendars'].keys()) < 1:
         logger.fatal('Found no calendar section in the config file')
         raise InvalidSettingsError()
@@ -211,38 +235,26 @@ def config_checks(
             config['locale']['local_timezone'])
 
     # expand calendars with type = discover
-    vdirs_complete = list()
-    vdir_colors_from_config = {}
-    for calendar in list(config['calendars'].keys()):
-        if not isinstance(config['calendars'][calendar], dict):
+    # we need a copy of config['calendars'], because we modify config in the body of the loop
+    for cname, cconfig in sorted(config['calendars'].items()):
+        if not isinstance(config['calendars'][cname], dict):
             logger.fatal('Invalid config file, probably missing calendar sections')
             raise InvalidSettingsError
-        if config['calendars'][calendar]['type'] == 'discover':
-            logger.debug(
-                f"discovering calendars in {config['calendars'][calendar]['path']}"
-            )
-            vdirs = get_all_vdirs(config['calendars'][calendar]['path'])
-            vdirs_complete += vdirs
-            if 'color' in config['calendars'][calendar]:
-                for vdir in vdirs:
-                    vdir_colors_from_config[vdir] = config['calendars'][calendar]['color']
-            config['calendars'].pop(calendar)
-    for vdir in sorted(vdirs_complete):
-        calendar = {'path': vdir,
-                    'color': _get_color_from_vdir(vdir),
+        if config['calendars'][cname]['type'] == 'discover':
+            logger.debug(f"discovering calendars in {cconfig['path']}")
+            vdirs_discovered = get_all_vdirs(cconfig['path'])
+            logger.debug(f"found the following vdirs: {vdirs_discovered}")
+            for vdir in vdirs_discovered:
+                vdir_config = {
+                    'path': vdir,
+                    'color': _get_color_from_vdir(vdir) or cconfig.get('color', None),
                     'type': _get_vdir_type(vdir),
-                    'readonly': False,
+                    'readonly': cconfig.get('readonly', False),
                     'priority': 10,
-                    }
-
-        # get color from config if not defined in vdir
-
-        if calendar['color'] is None and vdir in vdir_colors_from_config:
-            logger.debug(f"using collection's color for {vdir}")
-            calendar['color'] = vdir_colors_from_config[vdir]
-
-        name = get_unique_name(vdir, config['calendars'].keys())
-        config['calendars'][name] = calendar
+                }
+                unique_vdir_name = get_unique_name(vdir, config['calendars'].keys())
+                config['calendars'][unique_vdir_name] = vdir_config
+            config['calendars'].pop(cname)
 
     test_default_calendar(config)
     for calendar in config['calendars']:
