@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -22,36 +22,36 @@
 import calendar
 import datetime as dt
 from locale import LC_ALL, LC_TIME, getlocale, setlocale
+from typing import List, Optional, Union
 
 from click import style
 
+from .khalendar import CalendarCollection
 from .terminal import colored
 from .utils import get_month_abbr_len
 
 setlocale(LC_ALL, '')
 
 
-def get_weekheader(firstweekday):
+def get_weekheader(firstweekday: int) -> str:
     try:
-        mylocale = '.'.join(getlocale(LC_TIME))
+        mylocale = '.'.join(getlocale(LC_TIME))  # type: ignore
     except TypeError:
         mylocale = 'C'
 
-    _calendar = calendar.LocaleTextCalendar(firstweekday, locale=mylocale)
+    _calendar = calendar.LocaleTextCalendar(firstweekday, locale=mylocale)  # type: ignore
     return _calendar.formatweekheader(2)
 
 
-def getweeknumber(date):
+def getweeknumber(date: dt.date) -> int:
     """return iso week number for datetime.date object
     :param date: date
-    :type date: datetime.date()
     :return: weeknumber
-    :rtype: int
     """
     return dt.date.isocalendar(date)[1]
 
 
-def get_calendar_color(calendar, default_color, collection):
+def get_calendar_color(calendar: str, default_color: str, collection: CalendarCollection) -> str:
     """Because multi-line lambdas would be un-Pythonic
     """
     if collection._calendars[calendar]['color'] == '':
@@ -59,17 +59,44 @@ def get_calendar_color(calendar, default_color, collection):
     return collection._calendars[calendar]['color']
 
 
+def get_color_list(
+    calendars: List[str],
+    default_color: str,
+    collection: CalendarCollection
+) -> List[str]:
+    """Get the list of possible colors for the day, taking into account priority"""
+    dcolors = [
+        (
+            get_calendar_color(x, default_color, collection),
+            collection._calendars[x]["priority"],
+        )
+        for x in calendars
+    ]
+
+    dcolors.sort(key=lambda x: x[1], reverse=True)
+
+    maxPriority = dcolors[0][1]
+    return list({x[0] for x in filter(lambda x: x[1] == maxPriority, dcolors)})
+
+
 def str_highlight_day(
-        day, calendars, hmethod, default_color, multiple, color, bold_for_light_color, collection):
+    day: dt.date,
+    calendars: List[str],
+    hmethod: Optional[str],
+    default_color: str,
+    multiple: str,
+    multiple_on_overflow: bool,
+    color: str,
+    bold_for_light_color: bool,
+    collection: CalendarCollection,
+) -> str:
     """returns a string with day highlighted according to configuration
     """
     dstr = str(day.day).rjust(2)
     if color == '':
-        dcolors = list(set(
-            map(lambda x: get_calendar_color(x, default_color, collection), calendars)
-        ))
+        dcolors = get_color_list(calendars, default_color, collection)
         if len(dcolors) > 1:
-            if multiple == '':
+            if multiple == '' or (multiple_on_overflow and len(dcolors) == 2):
                 if hmethod == "foreground" or hmethod == "fg":
                     return colored(dstr[:1], fg=dcolors[0],
                                    bold_for_light_color=bold_for_light_color) + \
@@ -92,71 +119,83 @@ def str_highlight_day(
     return dstr
 
 
-def str_week(week, today, collection=None,
-             hmethod=None, default_color=None, multiple=None, color=None,
-             highlight_event_days=False, locale=None, bold_for_light_color=True):
+def str_week(
+    week: List[dt.date],
+    today: dt.date,
+    collection: Optional[CalendarCollection]=None,
+    hmethod: Optional[str]=None,
+    default_color: str='',
+    multiple: str='',
+    multiple_on_overflow: bool=False,
+    color: str='',
+    highlight_event_days: bool=False,
+    locale=None,
+    bold_for_light_color=True,
+) -> str:
     """returns a string representing one week,
     if for day == today color is reversed
 
     :param week: list of 7 datetime.date objects (one week)
-    :type day: list()
     :param today: the date of today
-    :type today: datetime.date
     :return: string, which if printed on terminal appears to have length 20,
              but may contain ascii escape sequences
-    :rtype: str
     """
     strweek = ''
+    if highlight_event_days and collection is None:
+        raise ValueError(
+            'if `highlight_event_days` is True, `collection` must be a CalendarCollection'
+        )
     for day in week:
         if day == today:
-            day = style(str(day.day).rjust(2), reverse=True)
+            day_str = style(str(day.day).rjust(2), reverse=True)
         elif highlight_event_days:
+            assert collection is not None
             devents = list(collection.get_calendars_on(day))
             if len(devents) > 0:
-                day = str_highlight_day(day, devents, hmethod, default_color,
-                                        multiple, color, bold_for_light_color, collection)
+                day_str = str_highlight_day(
+                    day, devents, hmethod, default_color, multiple,
+                    multiple_on_overflow, color, bold_for_light_color,
+                    collection,
+                )
             else:
-                day = str(day.day).rjust(2)
+                day_str = str(day.day).rjust(2)
         else:
-            day = str(day.day).rjust(2)
-        strweek = strweek + day + ' '
+            day_str = str(day.day).rjust(2)
+        strweek = strweek + day_str + ' '
     return strweek
 
 
-def vertical_month(month=None,
-                   year=None,
-                   today=None,
-                   weeknumber=False,
-                   count=3,
-                   firstweekday=0,
-                   monthdisplay='firstday',
+def vertical_month(month: Optional[int]=None,
+                   year: Optional[int]=None,
+                   today: Optional[dt.date]=None,
+                   weeknumber: Union[bool, str]=False,
+                   count: int=3,
+                   firstweekday: int=0,
+                   monthdisplay: str='firstday',
                    collection=None,
-                   hmethod='fg',
-                   default_color='',
-                   multiple='',
-                   color='',
+                   hmethod: str='fg',
+                   default_color: str='',
+                   multiple: str='',
+                   multiple_on_overflow: bool=False,
+                   color: str='',
                    highlight_event_days=False,
                    locale=None,
-                   bold_for_light_color=True):
+                   bold_for_light_color=True,
+                   ) -> List[str]:
     """
     returns a list() of str() of weeks for a vertical arranged calendar
 
     :param month: first month of the calendar,
                   if non given, current month is assumed
-    :type month: int
     :param year: year of the first month included,
                  if non given, current year is assumed
-    :type year: int
     :param today: day highlighted, if non is given, current date is assumed
-    :type today: datetime.date()
     :param weeknumber: if not False the iso weeknumber will be shown for each
                        week, if weeknumber is 'right' it will be shown in its
                        own column, if it is 'left' it will be shown interleaved
                        with the month names
-    :type weeknumber: str/bool
     :returns: calendar strings,  may also include some
               ANSI (color) escape strings
-    :rtype: list() of str()
     """
     if month is None:
         month = dt.date.today().month
@@ -165,7 +204,7 @@ def vertical_month(month=None,
     if today is None:
         today = dt.date.today()
 
-    khal = list()
+    khal = []
     w_number = '  ' if weeknumber == 'right' else ''
     calendar.setfirstweekday(firstweekday)
     weekheaders = get_weekheader(firstweekday)
@@ -179,7 +218,8 @@ def vertical_month(month=None,
             else:
                 new_month = len(week if week[0].day <= 7 else [])
             strweek = str_week(week, today, collection, hmethod, default_color,
-                               multiple, color, highlight_event_days, locale, bold_for_light_color)
+                               multiple, multiple_on_overflow, color, highlight_event_days, locale,
+                               bold_for_light_color)
             if new_month:
                 m_name = style(calendar.month_abbr[week[6].month].ljust(month_abbr_len), bold=True)
             elif weeknumber == 'left':
@@ -187,7 +227,7 @@ def vertical_month(month=None,
             else:
                 m_name = ' ' * month_abbr_len
             if weeknumber == 'right':
-                w_number = style('{:2}'.format(getweeknumber(week[0])), bold=True)
+                w_number = style(f'{getweeknumber(week[0]):2}', bold=True)
             else:
                 w_number = ''
 
