@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -22,6 +22,7 @@
 import datetime as dt
 import logging
 import os
+import stat
 import sys
 import textwrap
 from shutil import get_terminal_size
@@ -31,14 +32,13 @@ import click_log
 
 from . import __version__, controllers, khalendar
 from .exceptions import FatalError
-from .settings import InvalidSettingsError, get_config
-from .settings.exceptions import NoConfigFile
+from .settings import InvalidSettingsError, NoConfigFile, get_config
 from .terminal import colored
 
 try:
     from setproctitle import setproctitle
 except ImportError:
-    def setproctitle(x):
+    def setproctitle(_):
         pass
 
 
@@ -69,8 +69,8 @@ def multi_calendar_select(ctx, include_calendars, exclude_calendars):
         for cal_name in include_calendars:
             if cal_name not in ctx.obj['conf']['calendars']:
                 raise click.BadParameter(
-                    'Unknown calendar {}, run `khal printcalendars` to get a '
-                    'list of all configured calendars.'.format(cal_name)
+                    f'Unknown calendar {cal_name}, run `khal printcalendars` '
+                    'to get a list of all configured calendars.'
                 )
 
         selection.update(include_calendars)
@@ -106,8 +106,8 @@ def _select_one_calendar_callback(ctx, option, calendar):
 def _calendar_select_callback(ctx, option, calendar):
     if calendar and calendar not in ctx.obj['conf']['calendars']:
         raise click.BadParameter(
-            'Unknown calendar {}, run `khal printcalendars` to get a '
-            'list of all configured calendars.'.format(calendar)
+            f'Unknown calendar {calendar}, run `khal printcalendars` to get a '
+            'list of all configured calendars.'
         )
     return calendar
 
@@ -154,7 +154,7 @@ def global_options(f):
 def build_collection(conf, selection):
     """build and return a khalendar.CalendarCollection from the configuration"""
     try:
-        props = dict()
+        props = {}
         for name, cal in conf['calendars'].items():
             if selection is None or name in selection:
                 props[name] = {
@@ -162,6 +162,7 @@ def build_collection(conf, selection):
                     'path': cal['path'],
                     'readonly': cal['readonly'],
                     'color': cal['color'],
+                    'priority': cal['priority'],
                     'ctype': cal['type'],
                 }
         collection = khalendar.CalendarCollection(
@@ -172,6 +173,7 @@ def build_collection(conf, selection):
             hmethod=conf['highlight_days']['method'],
             default_color=conf['highlight_days']['default_color'],
             multiple=conf['highlight_days']['multiple'],
+            multiple_on_overflow=conf['highlight_days']['multiple_on_overflow'],
             highlight_event_days=conf['default']['highlight_event_days'],
         )
     except FatalError as error:
@@ -183,7 +185,7 @@ def build_collection(conf, selection):
     return collection
 
 
-class _NoConfig(object):
+class _NoConfig:
     def __getitem__(self, key):
         logger.fatal(
             'Cannot find a config file. If you have no configuration file '
@@ -213,16 +215,16 @@ def prepare_context(ctx, config):
 def stringify_conf(conf):
     # since we have only two levels of recursion, a recursive function isn't
     # really worth it
-    out = list()
+    out = []
     for key, value in conf.items():
-        out.append('[{}]'.format(key))
+        out.append(f'[{key}]')
         for subkey, subvalue in value.items():
             if isinstance(subvalue, dict):
-                out.append('  [[{}]]'.format(subkey))
+                out.append(f'  [[{subkey}]]')
                 for subsubkey, subsubvalue in subvalue.items():
-                    out.append('    {}: {}'.format(subsubkey, subsubvalue))
+                    out.append(f'    {subsubkey}: {subsubvalue}')
             else:
-                out.append('  {}: {}'.format(subkey, subvalue))
+                out.append(f'  {subkey}: {subvalue}')
     return '\n'.join(out)
 
 
@@ -277,6 +279,7 @@ def _get_cli():
                 hmethod=ctx.obj['conf']['highlight_days']['method'],
                 default_color=ctx.obj['conf']['highlight_days']['default_color'],
                 multiple=ctx.obj['conf']['highlight_days']['multiple'],
+                multiple_on_overflow=ctx.obj['conf']['highlight_days']['multiple_on_overflow'],
                 color=ctx.obj['conf']['highlight_days']['color'],
                 highlight_event_days=ctx.obj['conf']['default']['highlight_event_days'],
                 bold_for_light_color=ctx.obj['conf']['view']['bold_for_light_color'],
@@ -321,7 +324,11 @@ def _get_cli():
                 conf=ctx.obj['conf'],
                 env={"calendars": ctx.obj['conf']['calendars']}
             )
-            click.echo('\n'.join(event_column))
+            if event_column:
+                click.echo('\n'.join(event_column))
+            else:
+                logger.debug('No events found')
+
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
@@ -334,7 +341,7 @@ def _get_cli():
     @click.option('--location', '-l',
                   help=('The location of the new event.'))
     @click.option('--categories', '-g',
-                  help=('The categories of the new event.'))
+                  help=('The categories of the new event, comma separated.'))
     @click.option('--repeat', '-r',
                   help=('Repeat event: daily, weekly, monthly or yearly.'))
     @click.option('--until', '-u',
@@ -343,10 +350,12 @@ def _get_cli():
                   help=('The format to print the event.'))
     @click.option('--alarms', '-m',
                   help=('Alarm times for the new event as DELTAs comma separated'))
+    @click.option('--url', help=("URI for the event."))
     @click.argument('info', metavar='[START [END | DELTA] [TIMEZONE] [SUMMARY] [:: DESCRIPTION]]',
                     nargs=-1)
     @click.pass_context
-    def new(ctx, calendar, info, location, categories, repeat, until, alarms, format, interactive):
+    def new(ctx, calendar, info, location, categories, repeat, until, alarms, url, format,
+            interactive):
         '''Create a new event from arguments.
 
         START and END can be either dates, times or datetimes, please have a
@@ -356,10 +365,9 @@ def _get_cli():
         everything behind them is taken as the event's description.
         '''
         if not info and not interactive:
-                raise click.BadParameter(
-                    'no details provided, '
-                    'did you mean to use --interactive/-i?'
-                )
+            raise click.BadParameter(
+                'no details provided, did you mean to use --interactive/-i?'
+            )
 
         calendar = calendar or ctx.obj['conf']['default']['default_calendar']
         if calendar is None:
@@ -393,6 +401,7 @@ def _get_cli():
                 env={"calendars": ctx.obj['conf']['calendars']},
                 until=until,
                 alarms=alarms,
+                url=url,
                 format=format,
             )
         except FatalError as error:
@@ -431,20 +440,27 @@ def _get_cli():
             raise click.UsageError(
                 'When using batch import, please specify a calendar to import '
                 'into or set the `default_calendar` in the config file.')
+        rvalue = 0
+        # Default to stdin:
+        if not ics:
+            ics_strs = ((sys.stdin.read(), 'stdin'),)
+            if not batch:
 
-        try:
-            # Default to stdin:
-            if not ics:
-                ics_strs = (sys.stdin.read(),)
-                if not batch:
-                    if os.path.isfile('/dev/tty'):
-                        sys.stdin = open('/dev/tty', 'r')
-                    else:
-                        logger.warning('/dev/tty does not exist, importing might not work')
-            else:
-                ics_strs = (ics_file.read() for ics_file in ics)
+                def isatty(_file):
+                    try:
+                        return _file.isatty()
+                    except Exception:
+                        return False
 
-            for ics_str in ics_strs:
+                if isatty(sys.stdin) and os.stat('/dev/tty').st_mode & stat.S_IFCHR > 0:
+                    sys.stdin = open('/dev/tty')
+                else:
+                    logger.warning('/dev/tty does not exist, importing might not work')
+        else:
+            ics_strs = ((ics_file.read(), ics_file.name) for ics_file in ics)
+
+        for ics_str, filename in ics_strs:
+            try:
                 controllers.import_ics(
                     collection,
                     ctx.obj['conf'],
@@ -453,10 +469,14 @@ def _get_cli():
                     random_uid=random_uid,
                     env={"calendars": ctx.obj['conf']['calendars']},
                 )
-        except FatalError as error:
-            logger.debug(error, exc_info=True)
-            logger.fatal(error)
-            sys.exit(1)
+            except FatalError as error:
+                logger.debug(error, exc_info=True)
+                logger.fatal(f"An error occurred when trying to import the file from {filename}")
+                logger.fatal("Events from it will not be available in khal")
+                if not batch:
+                    sys.exit(1)
+                rvalue = 1
+        sys.exit(rvalue)
 
     @cli.command()
     @multi_calendar_option
@@ -464,7 +484,10 @@ def _get_cli():
     def interactive(ctx, include_calendar, exclude_calendar):
         '''Interactive UI. Also launchable via `ikhal`.'''
         controllers.interactive(
-            build_collection(ctx.obj['conf'], ctx.obj.get('calendar_selection', None)),
+            build_collection(
+                ctx.obj['conf'],
+                multi_calendar_select(ctx, include_calendar, exclude_calendar)
+            ),
             ctx.obj['conf']
         )
 
@@ -503,16 +526,16 @@ def _get_cli():
     def printformats(ctx):
         '''Print a date in all formats.
 
-        Print the date 2013-12-21 10:09 in all configured date(time)
+        Print the date 2013-12-21 21:45 in all configured date(time)
         formats to check if these locale settings are configured to ones
         liking.'''
-        time = dt.datetime(2013, 12, 21, 10, 9)
+        time = dt.datetime(2013, 12, 21, 21, 45)
         try:
             for strftime_format in [
                     'longdatetimeformat', 'datetimeformat', 'longdateformat',
                     'dateformat', 'timeformat']:
                 dt_str = time.strftime(ctx.obj['conf']['locale'][strftime_format])
-                click.echo('{}: {}'.format(strftime_format, dt_str))
+                click.echo(f'{strftime_format}: {dt_str}')
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
@@ -561,7 +584,7 @@ def _get_cli():
                 multi_calendar_select(ctx, include_calendar, exclude_calendar)
             )
             events = sorted(collection.search(search_string))
-            event_column = list()
+            event_column = []
             term_width, _ = get_terminal_size()
             now = dt.datetime.now()
             env = {"calendars": ctx.obj['conf']['calendars']}
@@ -572,7 +595,10 @@ def _get_cli():
                              bold_for_light_color=ctx.obj['conf']['view']['bold_for_light_color'])
                      for d in desc]
                 )
-            click.echo('\n'.join(event_column))
+            if event_column:
+                click.echo('\n'.join(event_column))
+            else:
+                logger.debug('No events found')
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
@@ -619,6 +645,8 @@ def _get_cli():
         '''Print all events at a specific datetime (defaults to now).'''
         if not datetime:
             datetime = ("now",)
+        if format is None:
+            format = ctx.obj['conf']['view']['event_format']
         try:
             rows = controllers.khal_list(
                 build_collection(
@@ -633,7 +661,8 @@ def _get_cli():
                 conf=ctx.obj['conf'],
                 env={"calendars": ctx.obj['conf']['calendars']}
             )
-            click.echo('\n'.join(rows))
+            if rows:
+                click.echo('\n'.join(rows))
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
