@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -22,11 +22,14 @@
 """collection of icalendar helper functions"""
 
 import datetime as dt
+import logging
+from collections import defaultdict
+from hashlib import sha256
+from typing import List, Optional, Tuple, Union
+
 import dateutil.rrule
 import icalendar
-import logging
 import pytz
-from collections import defaultdict
 
 from .exceptions import UnsupportedRecurrence
 from .parse_datetime import guesstimedeltafstr, rrulefstr
@@ -35,14 +38,12 @@ from .utils import generate_random_uid, localize_strip_tz, to_unix_time
 logger = logging.getLogger('khal')
 
 
-def split_ics(ics, random_uid=False, default_timezone=None):
+def split_ics(ics: str, random_uid: bool=False, default_timezone=None):
     """split an ics string into several according to VEVENT's UIDs
 
     and sort the right VTIMEZONEs accordingly
     ignores all other ics components
-    :type ics: str
     :param random_uid: assign random uids to all events
-    :type random_uid: bool
     :rtype list:
     """
     cal = cal_from_ics(ics)
@@ -62,43 +63,60 @@ def split_ics(ics, random_uid=False, default_timezone=None):
             tzs[key] = item
 
         if item.name == 'VEVENT':
+            if 'UID' not in item:
+                logger.warning(
+                    f"Event with summary '{item['SUMMARY']}' doesn't have a unique ID."
+                    "A generated ID will be used instead."
+                )
+                item['UID'] = sha256(item.to_ical()).hexdigest()
             events_grouped[item['UID']].append(item)
         else:
             continue
-    return [ics_from_list(events, tzs, random_uid, default_timezone) for uid, events in
-            sorted(events_grouped.items())]
+    out = []
+    saved_exception = None
+    for uid, events in sorted(events_grouped.items()):
+        try:
+            ics = ics_from_list(events, tzs, random_uid, default_timezone)
+        except Exception as exception:
+            logger.warn(f'Error when trying to import the event {uid}')
+            saved_exception = exception
+        else:
+            out.append(ics)
+    if saved_exception:
+        raise saved_exception
+    return out
 
 
-def new_event(locale, dtstart=None, dtend=None, summary=None, timezone=None,
-              allday=False, description=None, location=None, categories=None,
-              repeat=None, until=None, alarms=None):
+def new_vevent(locale,
+               dtstart: dt.date,
+               dtend: dt.date,
+               summary: str,
+               timezone: Optional[pytz.BaseTzInfo]=None,
+               allday: bool=False,
+               description: Optional[str]=None,
+               location: Optional[str]=None,
+               categories: Optional[Union[List[str], str]]=None,
+               repeat: Optional[str]=None,
+               until=None,
+               alarms: Optional[str]=None,
+               url: Optional[str]=None,
+               ) -> icalendar.Event:
     """create a new event
 
     :param dtstart: starttime of that event
-    :type dtstart: datetime
     :param dtend: end time of that event, if this is a *date*, this value is
         interpreted as being the last date the event is scheduled on, i.e.
         the VEVENT DTEND will be *one day later*
-    :type dtend: datetime
     :param summary: description of the event, used in the SUMMARY property
-    :type summary: unicode
     :param timezone: timezone of the event (start and end)
-    :type timezone: pytz.timezone
     :param allday: if set to True, we will not transform dtstart and dtend to
         datetime
-    :type allday: bool
+    :param url: url of the event
     :returns: event
-    :rtype: icalendar.Event
     """
-
-    if dtstart is None:
-        raise ValueError("no start given")
-    if dtend is None:
-        raise ValueError("no end given")
-    if summary is None:
-        raise ValueError("no summary given")
-
     if not allday and timezone is not None:
+        assert isinstance(dtstart, dt.datetime)
+        assert isinstance(dtend, dt.datetime)
         dtstart = timezone.localize(dtstart)
         dtend = timezone.localize(dtend)
 
@@ -116,8 +134,10 @@ def new_event(locale, dtstart=None, dtend=None, summary=None, timezone=None,
         event.add('location', location)
     if categories:
         event.add('categories', categories)
+    if url:
+        event.add('url', icalendar.vUri(url))
     if repeat and repeat != "none":
-        rrule = rrulefstr(repeat, until, locale)
+        rrule = rrulefstr(repeat, until, locale, getattr(dtstart, 'tzinfo', None))
         event.add('rrule', rrule)
     if alarms:
         for alarm in alarms.split(","):
@@ -131,13 +151,16 @@ def new_event(locale, dtstart=None, dtend=None, summary=None, timezone=None,
     return event
 
 
-def ics_from_list(events, tzs, random_uid=False, default_timezone=None):
-    """convert an iterable of icalendar.Events to an icalendar.Calendar
+def ics_from_list(
+    events: List[icalendar.Event],
+    tzs,
+    random_uid: bool=False,
+    default_timezone=None
+) -> str:
+    """convert an iterable of icalendar.Events to an icalendar str
 
     :params events: list of events all with the same uid
-    :type events: list(icalendar.cal.Event)
     :param random_uid: assign random uids to all events
-    :type random_uid: bool
     :param tzs: collection of timezones
     :type tzs: dict(icalendar.cal.Vtimzone
     """
@@ -174,8 +197,8 @@ def ics_from_list(events, tzs, random_uid=False, default_timezone=None):
                 if datetime_.tzinfo is None and 'TZID' in item.params and \
                         item.params['TZID'] not in missing_tz:
                     logger.warning(
-                        'Cannot find timezone `{}` in .ics file, using default timezone. '
-                        'This can lead to erroneous time shifts'.format(item.params['TZID'])
+                        f"Cannot find timezone `{item.params['TZID']}` in .ics file, "
+                        "using default timezone. This can lead to erroneous time shifts"
                     )
                     missing_tz.add(item.params['TZID'])
                 elif datetime_.tzinfo and datetime_.tzinfo != pytz.UTC and \
@@ -187,8 +210,8 @@ def ics_from_list(events, tzs, random_uid=False, default_timezone=None):
             calendar.add_component(tzs[str(tzid)])
         else:
             logger.warning(
-                'Cannot find timezone `{}` in .ics file, this could be a bug, '
-                'please report this issue at http://github.com/pimutils/khal/.'.format(tzid))
+                f'Cannot find timezone `{tzid}` in .ics file, this could be a bug, '
+                'please report this issue at http://github.com/pimutils/khal/.')
     for sub_event in events:
         calendar.add_component(sub_event)
     return calendar.to_ical().decode('utf-8')
@@ -257,13 +280,35 @@ def expand(vevent, href=''):
             # if python can deal with larger datetime values yet and b) pytz
             # doesn't know any larger transition times
             rrule._until = dt.datetime(2037, 12, 31)
-        elif events_tz and 'Z' in rrule_param.to_ical().decode():
-            rrule._until = pytz.UTC.localize(
-                rrule._until).astimezone(events_tz).replace(tzinfo=None)
+        else:
+            if events_tz and 'Z' in rrule_param.to_ical().decode():
+                rrule._until = pytz.UTC.localize(
+                    rrule._until).astimezone(events_tz).replace(tzinfo=None)
+
+            # rrule._until and dtstart could be dt.date or dt.datetime. They
+            # need to be the same for comparison
+            testuntil = rrule._until
+            if (type(dtstart) == dt.date and type(testuntil) == dt.datetime):
+                testuntil = testuntil.date()
+            teststart = dtstart
+            if (type(testuntil) == dt.date and type(teststart) == dt.datetime):
+                teststart = teststart.date()
+
+            if testuntil < teststart:
+                logger.warning(
+                    f'{href}: Unsupported recurrence. UNTIL is before DTSTART.\n'
+                    'This event will not be available in khal.')
+                return False
+
+        if rrule.count() == 0:
+            logger.warning(
+                f'{href}: Recurrence defined but will never occur.\n'
+                'This event will not be available in khal.')
+            return False
 
         rrule = map(sanitize_datetime, rrule)
 
-        logger.debug('calculating recurrence dates for {}, this might take some time.'.format(href))
+        logger.debug(f'calculating recurrence dates for {href}, this might take some time.')
 
         # RRULE and RDATE may specify the same date twice, it is recommended by
         # the RFC to consider this as only one instance
@@ -296,8 +341,8 @@ def expand(vevent, href=''):
                 dtstartl.remove(date)
             except KeyError:
                 logger.warning(
-                    'In event {}, excluded instance starting at {} not found, '
-                    'event might be invalid.'.format(href, date))
+                    f'In event {href}, excluded instance starting at {date} '
+                    'not found, event might be invalid.')
 
     dtstartend = [(start, start + duration) for start in dtstartl]
     # not necessary, but I prefer deterministic output
@@ -306,7 +351,7 @@ def expand(vevent, href=''):
 
 
 def assert_only_one_uid(cal: icalendar.Calendar):
-    """assert the all VEVENTs in cal have the same UID"""
+    """assert that all VEVENTs in cal have the same UID"""
     uids = set()
     for item in cal.walk():
         if item.name == 'VEVENT':
@@ -345,9 +390,9 @@ def sanitize(vevent, default_timezone, href='', calendar=''):
             value = default_timezone.localize(vevent.pop(prop).dt)
             vevent.add(prop, value)
             logger.warning(
-                "{} localized in invalid or incomprehensible timezone `{}` in {}/{}. "
-                "This could lead to this event being wrongly displayed."
-                "".format(prop, timezone, calendar, href)
+                f"{prop} localized in invalid or incomprehensible timezone "
+                f"`{timezone}` in {calendar}/{href}. This could lead to this "
+                "event being wrongly displayed."
             )
 
     vdtstart = vevent.pop('DTSTART', None)
@@ -384,11 +429,15 @@ def sanitize_timerange(dtstart, dtend, duration=None):
                 "Assuming it's the same timezone as the end time"
             )
             dtstart = dtend.tzinfo.localize(dtstart)
+    if dtend is not None and type(dtstart) != type(dtend):
+        raise ValueError(
+            'The event\'s end time (DTEND) and start time (DTSTART) are not of the same type.')
 
     if dtend is None and duration is None:
         if isinstance(dtstart, dt.datetime):
-            dtstart = dtstart.date()
-        dtend = dtstart + dt.timedelta(days=1)
+            dtend = dtstart + dt.timedelta(hours=1)
+        else:
+            dtend = dtstart + dt.timedelta(days=1)
     elif dtend is not None:
         if dtend < dtstart:
             raise ValueError('The event\'s end time (DTEND) is older than '
@@ -398,7 +447,10 @@ def sanitize_timerange(dtstart, dtend, duration=None):
                 "Event start time and end time are the same. "
                 "Assuming the event's duration is one hour."
             )
-            dtend += dt.timedelta(hours=1)
+            if isinstance(dtstart, dt.datetime):
+                dtend += dt.timedelta(hours=1)
+            else:
+                dtend += dt.timedelta(days=1)
 
     return dtstart, dtend
 
@@ -435,7 +487,7 @@ def _get_all_properties(vevent, prop):
     :type prop: str
     """
     if prop not in vevent:
-        return list()
+        return []
     if isinstance(vevent[prop], list):
         rdates = [leaf.dt for tree in vevent[prop] for leaf in tree.dts]
     else:
@@ -443,12 +495,9 @@ def _get_all_properties(vevent, prop):
     return rdates
 
 
-def delete_instance(vevent, instance):
+def delete_instance(vevent: icalendar.Event, instance: dt.datetime) -> None:
     """remove a recurrence instance from a VEVENT's RRDATE list or add it
     to the EXDATE list
-
-    :type vevent: icalendar.cal.Event
-    :type instance: datetime.datetime
     """
     # TODO check where this instance is coming from and only call the
     # appropriate function
@@ -464,7 +513,7 @@ def delete_instance(vevent, instance):
             vevent.add('RDATE', rdates)
 
 
-def sort_key(vevent):
+def sort_key(vevent: icalendar.Event) -> Tuple[str, float]:
     """helper function to determine order of VEVENTS
     so that recurrence-id events come after the corresponding rrule event, etc
     :param vevent: icalendar.Event
@@ -482,7 +531,10 @@ def sort_key(vevent):
         return uid, 1
 
 
-def cal_from_ics(ics):
+def cal_from_ics(ics: str) -> icalendar.Calendar:
+    """
+    :param ics: an icalendar formatted string
+    """
     try:
         cal = icalendar.Calendar.from_ical(ics)
     except ValueError as error:
@@ -495,4 +547,6 @@ def cal_from_ics(ics):
             icalendar.vUTCOffset.ignore_exceptions = True
             cal = icalendar.Calendar.from_ical(ics)
             icalendar.vUTCOffset.ignore_exceptions = False
+        else:
+            raise
     return cal
