@@ -6,16 +6,29 @@ vdirsyncer.
 import errno
 import os
 import uuid
-
-from typing import Optional  # noqa
+from hashlib import sha1
+from typing import IO, Callable, Dict, Iterable, Optional, Protocol, Tuple, Type
 
 from atomicwrites import atomic_write
 
+from ..custom_types import PathLike, SupportsRaw
+
+
+class HasMetaProtocol(Protocol):
+    color_type: Callable
+
+    def get_meta(self, key: str) -> str:
+        ...
+
+    def set_meta(self, key: str, value: str) -> None:
+        ...
+
 
 class cached_property:
     '''A read-only @property that is only evaluated once. Only usable on class
     instances' methods.
     '''
+
     def __init__(self, fget, doc=None):
         self.__name__ = fget.__name__
         self.__module__ = fget.__module__
@@ -29,35 +42,25 @@ class cached_property:
         return result
 
 
-def to_unicode(x, encoding='ascii'):
-    if not isinstance(x, str):
-        return x.decode(encoding)
-    return x
-
-
-def to_bytes(x, encoding='ascii'):
-    if not isinstance(x, bytes):
-        return x.encode(encoding)
-    return x
-
-
 SAFE_UID_CHARS = ('abcdefghijklmnopqrstuvwxyz'
                   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-                  '0123456789_.-+')
+                  '0123456789_.-+@')
 
 
-def _href_safe(uid, safe=SAFE_UID_CHARS):
+def _href_safe(uid: str, safe: str=SAFE_UID_CHARS) -> bool:
     return not bool(set(uid) - set(safe))
 
 
-def _generate_href(uid=None, safe=SAFE_UID_CHARS):
-    if not uid or not _href_safe(uid, safe):
-        return to_unicode(uuid.uuid4().hex)
+def _generate_href(uid: Optional[str]=None, safe: str=SAFE_UID_CHARS) -> str:
+    if not uid:
+        return str(uuid.uuid4().hex)
+    elif not _href_safe(uid, safe):
+        return sha1(uid.encode()).hexdigest()
     else:
         return uid
 
 
-def get_etag_from_file(f):
+def get_etag_from_file(f) -> str:
     '''Get mtime-based etag from a filepath, file-like object or raw file
     descriptor.
 
@@ -87,17 +90,17 @@ def get_etag_from_file(f):
     mtime = getattr(stat, 'st_mtime_ns', None)
     if mtime is None:
         mtime = stat.st_mtime
-    return '{:.9f}'.format(mtime)
+    return f'{mtime:.9f}'
 
 
 class VdirError(IOError):
     def __init__(self, *args, **kwargs):
         for key, value in kwargs.items():
-            if getattr(self, key, object()) is not None:  # pragma: no cover
-                raise TypeError('Invalid argument: {}'.format(key))
+            if getattr(self, key, object()) not in [None, '']:  # pragma: no cover
+                raise TypeError(f'Invalid argument: {key}')
             setattr(self, key, value)
 
-        super(VdirError, self).__init__(*args)
+        super().__init__(*args)
 
 
 class NotFoundError(VdirError):
@@ -113,16 +116,16 @@ class WrongEtagError(VdirError):
 
 
 class AlreadyExistingError(VdirError):
-    existing_href = None  # type: Optional[str]
+    existing_href: str = ''
 
 
 class Item:
-    def __init__(self, raw):
+    def __init__(self, raw: str):
         assert isinstance(raw, str)
         self.raw = raw
 
     @cached_property
-    def uid(self):
+    def uid(self) -> Optional[str]:
         uid = ''
         lines = iter(self.raw.splitlines())
         for line in lines:
@@ -138,15 +141,11 @@ class Item:
         return uid or None
 
 
-def _normalize_meta_value(value):
-    return to_unicode(value or '').strip()
-
-
 class VdirBase:
     item_class = Item
     default_mode = 0o750
 
-    def __init__(self, path, fileext, encoding='utf-8'):
+    def __init__(self, path: str, fileext: str, encoding: str='utf-8'):
         if not os.path.isdir(path):
             raise CollectionNotFoundError(path)
         self.path = path
@@ -154,7 +153,7 @@ class VdirBase:
         self.fileext = fileext
 
     @classmethod
-    def discover(cls, path, **kwargs):
+    def discover(cls, path: str, **kwargs) -> Iterable['VdirBase']:
         try:
             collections = os.listdir(path)
         except OSError as e:
@@ -168,50 +167,52 @@ class VdirBase:
                 yield cls(path=collection_path, **kwargs)
 
     @classmethod
-    def create(cls, collection_name, **kwargs):
+    def create(cls, collection_name: PathLike, **kwargs: PathLike) -> Dict[str, PathLike]:
         kwargs = dict(kwargs)
         path = kwargs['path']
 
-        path = os.path.join(path, collection_name)
-        if not os.path.exists(path):
-            os.makedirs(path, mode=cls.default_mode)
-        elif not os.path.isdir(path):
-            raise IOError('{} is not a directory.'.format(repr(path)))
+        pathn = os.path.join(path, collection_name)
+        if not os.path.exists(pathn):
+            os.makedirs(pathn, mode=cls.default_mode)
+        elif not os.path.isdir(pathn):
+            raise OSError(f'{repr(pathn)} is not a directory.')
 
-        kwargs['path'] = path
+        kwargs['path'] = pathn
         return kwargs
 
-    def _get_filepath(self, href):
+    def _get_filepath(self, href: str) -> str:
         return os.path.join(self.path, href)
 
-    def _get_href(self, uid):
+    def _get_href(self, uid: Optional[str]) -> str:
         return _generate_href(uid) + self.fileext
 
-    def list(self):
+    def list(self) -> Iterable[Tuple[str, str]]:
         for fname in os.listdir(self.path):
             fpath = os.path.join(self.path, fname)
             if os.path.isfile(fpath) and fname.endswith(self.fileext):
                 yield fname, get_etag_from_file(fpath)
 
-    def get(self, href):
+    def get(self, href: str) -> Tuple[Item, str]:
         fpath = self._get_filepath(href)
         try:
             with open(fpath, 'rb') as f:
-                return (Item(f.read().decode(self.encoding)),
-                        get_etag_from_file(fpath))
-        except IOError as e:
+                return (
+                    Item(f.read().decode(self.encoding)),
+                    get_etag_from_file(fpath)
+                )
+        except OSError as e:
             if e.errno == errno.ENOENT:
                 raise NotFoundError(href)
             else:
                 raise
 
-    def upload(self, item):
+    def upload(self, item: SupportsRaw) -> Tuple[str, str]:
         if not isinstance(item.raw, str):
             raise TypeError('item.raw must be a unicode string.')
 
         try:
             href = self._get_href(item.uid)
-            fpath, etag = self._upload_impl(item, href)
+            _, etag = self._upload_impl(item, href)
         except OSError as e:
             if e.errno in (
                 errno.ENAMETOOLONG,  # Unix
@@ -219,15 +220,15 @@ class VdirBase:
             ):
                 # random href instead of UID-based
                 href = self._get_href(None)
-                fpath, etag = self._upload_impl(item, href)
+                _, etag = self._upload_impl(item, href)
             else:
                 raise
-
         return href, etag
 
-    def _upload_impl(self, item, href):
+    def _upload_impl(self, item: SupportsRaw, href: str) -> Tuple[str, str]:
         fpath = self._get_filepath(href)
         try:
+            f: IO
             with atomic_write(fpath, mode='wb', overwrite=False) as f:
                 f.write(item.raw.encode(self.encoding))
                 return fpath, get_etag_from_file(f)
@@ -237,7 +238,7 @@ class VdirBase:
             else:
                 raise
 
-    def update(self, href, item, etag):
+    def update(self, href: str, item: SupportsRaw, etag: str) -> str:
         fpath = self._get_filepath(href)
         if not os.path.exists(fpath):
             raise NotFoundError(item.uid)
@@ -254,7 +255,7 @@ class VdirBase:
 
         return etag
 
-    def delete(self, href, etag):
+    def delete(self, href: str, etag: Optional[str]) -> None:
         fpath = self._get_filepath(href)
         if not os.path.isfile(fpath):
             raise NotFoundError(href)
@@ -263,18 +264,18 @@ class VdirBase:
             raise WrongEtagError(etag, actual_etag)
         os.remove(fpath)
 
-    def get_meta(self, key):
+    def get_meta(self, key: str) -> Optional[str]:
         fpath = os.path.join(self.path, key)
         try:
             with open(fpath, 'rb') as f:
-                return f.read().decode(self.encoding) or None
-        except IOError as e:
+                return f.read().decode(self.encoding).strip() or None
+        except OSError as e:
             if e.errno == errno.ENOENT:
                 return None
             else:
                 raise
 
-    def set_meta(self, key, value):
+    def set_meta(self, key: str, value: str) -> None:
         value = value or ''
         assert isinstance(value, str)
         fpath = os.path.join(self.path, key)
@@ -283,7 +284,7 @@ class VdirBase:
 
 
 class Color:
-    def __init__(self, x):
+    def __init__(self, x: str):
         if not x:
             raise ValueError('Color is false-ish.')
         if not x.startswith('#'):
@@ -291,10 +292,10 @@ class Color:
         if len(x) != 7:
             raise ValueError('Color must not have shortcuts. '
                              '#ffffff instead of #fff')
-        self.raw = x.upper()
+        self.raw: str = x.upper()
 
     @cached_property
-    def rgb(self):
+    def rgb(self) -> Tuple[int, int, int]:
         x = self.raw
 
         r = x[1:3]
@@ -304,28 +305,27 @@ class Color:
         if len(r) == len(g) == len(b) == 2:
             return int(r, 16), int(g, 16), int(b, 16)
         else:
-            raise ValueError('Unable to parse color value: {}'
-                             .format(self.value))
+            raise ValueError(f'Unable to parse color value: {self.raw}')
 
 
-class ColorMixin:
-    color_type = Color
+class ColorMixin():
+    color_type: Type[Color] = Color
 
-    def get_color(self):
+    def get_color(self: HasMetaProtocol) -> Optional[str]:
         try:
             return self.color_type(self.get_meta('color'))
         except ValueError:
             return None
 
-    def set_color(self, value):
+    def set_color(self: HasMetaProtocol, value: str) -> None:
         self.set_meta('color', self.color_type(value).raw)
 
 
 class DisplayNameMixin:
-    def get_displayname(self):
+    def get_displayname(self: HasMetaProtocol) -> str:
         return self.get_meta('displayname')
 
-    def set_displayname(self, value):
+    def set_displayname(self: HasMetaProtocol, value: str) -> None:
         self.set_meta('displayname', value)
 
 
