@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2021 khal contributors
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -23,54 +23,54 @@ import datetime as dt
 import logging
 import signal
 import sys
+from typing import Optional
 
 import click
 import urwid
 
-from .. import icalendar as icalendar_helpers, utils
-from ..khalendar.event import Event
+from .. import utils
 from ..khalendar.exceptions import ReadOnlyCalendarError
 from . import colors
-from .widgets import ExtendedEdit as Edit, NPile, NColumns, linebox
 from .base import Pane, Window
 from .editor import EventEditor, ExportDialog
-from .calendarwidget import CalendarWidget
+from .widgets import CalendarWidget, NColumns, NPile, linebox
+from .widgets import ExtendedEdit as Edit
 
 logger = logging.getLogger('khal')
 
 #  Overview of how this all meant to fit together:
 #
-#   +--ClassicView(Pane)---------------------------------------+
-#   |                                                          |
-#   | +-CalendarWidget--+ +----EventColumn-------------------+ |
-#   | |                 | |                                  | |
-#   | |                 | | +-DListBox---------------------+ | |
-#   | |                 | | |                              | | |
-#   | |                 | | | +-DayWalker----------------+ | | |
-#   | |                 | | | |                          | | | |
-#   | |                 | | | | +-BoxAdapter-----------+ | | | |
-#   | |                 | | | | |                      | | | | |
-#   | |                 | | | | | +-DateListBox------+ | | | | |
-#   | |                 | | | | | | DateHeader       | | | | | |
-#   | |                 | | | | | | U_Event          | | | | | |
-#   | |                 | | | | | |  ...             | | | | | |
-#   | |                 | | | | | | U_Event          | | | | | |
-#   | |                 | | | | | +------------------+ | | | | |
-#   | |                 | | | | +----------------------+ | | | |
-#   | |                 | | | |       ...                | | | |
-#   | |                 | | | | +-BoxAdapter-----------+ | | | |
-#   | |                 | | | | |                      | | | | |
-#   | |                 | | | | | +-DateListBox------+ | | | | |
-#   | |                 | | | | | | DateHeader       | | | | | |
-#   | |                 | | | | | | U_Event          | | | | | |
-#   | |                 | | | | | |  ...             | | | | | |
-#   | |                 | | | | | | U_Event          | | | | | |
-#   | |                 | | | | | +------------------+ | | | | |
-#   | |                 | | | | +----------------------+ | | | |
-#   | |                 | | | +--------------------------+ | | |
-#   | |                 | | +------------------------------+ | |
-#   | +-----------------+ +----------------------------------+ |
-#   +----------------------------------------------------------+
+#   ┌──ClassicView(Pane)────────────────────────────────────────┐
+#   │                                                           │
+#   │ ┌─CalendarWidget──┐  ┌────EventColumn───────────────────┐ │
+#   │ │                 │  │                                  │ │
+#   │ │                 │  │ ┌─DListBox─────────────────────┐ │ │
+#   │ │                 │  │ │                              │ │ │
+#   │ │                 │  │ │ ┌─DayWalker────────────────┐ │ │ │
+#   │ │                 │  │ │ │                          │ │ │ │
+#   │ │                 │  │ │ │ ┌─BoxAdapter───────────┐ │ │ │ │
+#   │ │                 │  │ │ │ │                      │ │ │ │ │
+#   │ │                 │  │ │ │ │ ┌─DateListBox──────┐ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │ DateHeader       │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │ U_Event          │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │  ...             │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │ U_Event          │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ └──────────────────┘ │ │ │ │ │
+#   │ │                 │  │ │ │ └──────────────────────┘ │ │ │ │
+#   │ │                 │  │ │ │       ...                │ │ │ │
+#   │ │                 │  │ │ │ ┌─BoxAdapter───────────┐ │ │ │ │
+#   │ │                 │  │ │ │ │                      │ │ │ │ │
+#   │ │                 │  │ │ │ │ ┌─DateListBox──────┐ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │ DateHeader       │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │ U_Event          │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │  ...             │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ │ U_Event          │ │ │ │ │ │
+#   │ │                 │  │ │ │ │ └──────────────────┘ │ │ │ │ │
+#   │ │                 │  │ │ │ └──────────────────────┘ │ │ │ │
+#   │ │                 │  │ │ └──────────────────────────┘ │ │ │
+#   │ │                 │  │ └──────────────────────────────┘ │ │
+#   │ └─────────────────┘  └──────────────────────────────────┘ │
+#   └───────────────────────────────────────────────────────────┘
 
 ALL = 1
 INSTANCES = 2
@@ -108,7 +108,7 @@ class DateHeader(SelectableText):
         self._day = day
         self._dateformat = dateformat
         self._conf = conf
-        super().__init__('')
+        super().__init__('', wrap='clip')
         self.update_date_line()
 
     def update_date_line(self):
@@ -235,6 +235,7 @@ class U_Event(urwid.Text):
 
 class EventListBox(urwid.ListBox):
     """Container for list of U_Events"""
+
     def __init__(
             self, *args, parent, conf,
             delete_status, toggle_delete_instance, toggle_delete_all,
@@ -275,6 +276,7 @@ class EventListBox(urwid.ListBox):
 class DListBox(EventListBox):
     """Container for a DayWalker"""
     # XXX unfortunate naming, there is also DateListBox
+
     def __init__(self, *args, **kwargs):
         dynamic_days = kwargs.pop('dynamic_days', True)
         super().__init__(*args, **kwargs)
@@ -737,18 +739,17 @@ class EventColumn(urwid.WidgetWrap):
         assert not self.editor
         if external_edit:
             self.pane.window.loop.stop()
-            text = click.edit(event.raw)
+            ics = click.edit(event.raw)
             self.pane.window.loop.start()
-            if text is None:
+            if ics is None:
                 return
             # KeyErrors can occur here when we destroy DTSTART,
             # otherwise, even broken .ics files seem to be no problem
-            new_event = Event.fromString(
-                text,
-                locale=self._conf['locale'],
-                href=event.href,
-                calendar=event.calendar,
+            new_event = self.pane.collection.create_event_from_ics(
+                ics,
                 etag=event.etag,
+                calendar_name=event.calendar,
+                href=event.href,
             )
             self.pane.collection.update(new_event)
             update_colors(
@@ -847,7 +848,7 @@ class EventColumn(urwid.WidgetWrap):
         # up on disk but not be displayed in khal
         event = self.focus_event.event.duplicate()
         try:
-            self.pane.collection.new(event)
+            self.pane.collection.insert(event)
         except ReadOnlyCalendarError:
             event.calendar = self.pane.collection.default_calendar_name or \
                 self.pane.collection.writable_names[0]
@@ -863,34 +864,34 @@ class EventColumn(urwid.WidgetWrap):
         except IndexError:
             pass
 
-    def new(self, date, end=None):
+    def new(self, date: dt.date, end: Optional[dt.date]=None):
         """create a new event on `date` at the next full hour and edit it
 
         :param date: default date for new event
-        :type date: datetime.date
         :param end: optional, date the event ends on (inclusive)
-        :type end: datetime.date
         """
+        dtstart: dt.date
+        dtend: dt.date
         if not self.pane.collection.writable_names:
             self.pane.window.alert(('alert', 'No writable calendar.'))
             return
         if date is None:
             date = dt.datetime.now()
         if end is None:
-            start = dt.datetime.combine(date, dt.time(dt.datetime.now().hour))
-            end = start + dt.timedelta(minutes=60)
-            event = icalendar_helpers.new_event(
-                dtstart=start, dtend=end, summary='',
-                timezone=self._conf['locale']['default_timezone'],
-                locale=self._conf['locale'],
-            )
-        else:
-            event = icalendar_helpers.new_event(
-                dtstart=date, dtend=end + dt.timedelta(days=1), summary='',
-                allday=True, locale=self._conf['locale'],
-            )
-        event = self.pane.collection.new_event(
-            event.to_ical(), self.pane.collection.default_calendar_name)
+            dtstart = dt.datetime.combine(date, dt.time(dt.datetime.now().hour))
+            dtend = dtstart + dt.timedelta(minutes=60)
+            allday = False
+        else:  # TODO XXX why do we use allday if end is not set???
+            dtstart = date
+            dtend = end + dt.timedelta(days=1)
+            allday = True
+        event = self.pane.collection.create_event_from_dict({
+            'dtstart': dtstart,
+            'dtend': dtend,
+            'summary': '',
+            'timezone': self._conf['locale']['default_timezone'],
+            'allday': allday,
+        })
         self.edit(event)
 
     def selectable(self):
@@ -963,6 +964,11 @@ class EventDisplay(urwid.WidgetWrap):
         if event.url != '':
             lines.append(urwid.Text('URL: ' + event.url))
 
+        if event.attendees != '':
+            lines.append(urwid.Text('Attendees:'))
+            for attendee in event.attendees.split(', '):
+                lines.append(urwid.Text(f'  - {attendee}'))
+
         # start and end time/date
         if event.allday:
             startstr = event.start_local.strftime(self._conf['locale']['dateformat'])
@@ -997,6 +1003,7 @@ class EventDisplay(urwid.WidgetWrap):
 
 class SearchDialog(urwid.WidgetWrap):
     """A Search Dialog Widget"""
+
     def __init__(self, search_func, abort_func):
 
         class Search(Edit):
@@ -1107,9 +1114,7 @@ class ClassicView(Pane):
             self.collection.delete(href, etag, account)
         for part, rec_id in self._deleted[INSTANCES]:
             account, href, etag = part.split('\n', 2)
-            event = self.collection.get_event(href, account)
-            event.delete_instance(rec_id)
-            self.collection.update(event)
+            self.collection.delete_instance(href, etag, account, rec_id)
 
     def keypress(self, size, key):
         binds = self._conf['keybindings']
@@ -1288,7 +1293,7 @@ def start_pane(pane, callback, program_info='', quit_keys=None):
                 return 'DEBUG'
 
         def format(self, record):
-            return (self.get_prefix(record.levelno) + ': ' + record.msg)
+            return f'{self.get_prefix(record.levelno)}: {record.msg}'
 
     class HeaderFormatter(LogPaneFormatter):
         def format(self, record):
