@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2021 khal contributors
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -24,10 +24,20 @@ import datetime as dt
 import urwid
 
 from ..utils import get_weekday_occurrence, get_wrapped_text
-from .calendarwidget import CalendarWidget
-from .widgets import (AlarmsEditor, Choice, DateConversionError, DateWidget,
-                      ExtendedEdit, NColumns, NListBox, NPile, PositiveIntEdit,
-                      TimeWidget, ValidatedEdit)
+from .widgets import (
+    AlarmsEditor,
+    CalendarWidget,
+    Choice,
+    DateConversionError,
+    DateWidget,
+    ExtendedEdit,
+    NColumns,
+    NListBox,
+    NPile,
+    PositiveIntEdit,
+    TimeWidget,
+    ValidatedEdit,
+)
 
 
 class StartEnd:
@@ -48,7 +58,7 @@ class CalendarPopUp(urwid.PopUpLauncher):
         self._monthdisplay = monthdisplay
         self._firstweekday = firstweekday
         self._keybindings = {} if keybindings is None else keybindings
-        self.__super.__init__(widget)
+        super().__init__(widget)
 
     def keypress(self, size, key):
         if key == 'enter':
@@ -338,6 +348,7 @@ class EventEditor(urwid.WidgetWrap):
 
         self.description = event.description
         self.location = event.location
+        self.attendees = event.attendees
         self.categories = event.categories
         self.url = event.url
         self.startendeditor = StartEndEditor(
@@ -365,17 +376,25 @@ class EventEditor(urwid.WidgetWrap):
         )
         self.description = urwid.AttrMap(
             ExtendedEdit(
-                caption=('', 'Description: '),
+                caption=('', 'Description:  '),
                 edit_text=self.description,
                 multiline=True
             ),
             'edit'
         )
         self.location = urwid.AttrMap(ExtendedEdit(
-            caption=('', 'Location:    '), edit_text=self.location), 'edit'
+            caption=('', 'Location:     '), edit_text=self.location), 'edit'
         )
         self.categories = urwid.AttrMap(ExtendedEdit(
-            caption=('', 'Categories:  '), edit_text=self.categories), 'edit'
+            caption=('', 'Categories:   '), edit_text=self.categories), 'edit'
+        )
+        self.attendees = urwid.AttrMap(
+            ExtendedEdit(
+                caption=('', 'Attendees: '),
+                edit_text=self.attendees,
+                multiline=True
+            ),
+            'edit'
         )
         self.url = urwid.AttrMap(ExtendedEdit(
             caption=('', 'URL:         '), edit_text=self.url), 'edit'
@@ -390,6 +409,8 @@ class EventEditor(urwid.WidgetWrap):
             self.description,
             self.url,
             divider,
+            self.attendees,
+            divider,
             self.startendeditor,
             self.recurrenceeditor,
             divider,
@@ -428,6 +449,8 @@ class EventEditor(urwid.WidgetWrap):
             return True
         if get_wrapped_text(self.url) != self.event.url:
             return True
+        if get_wrapped_text(self.attendees) != self.event.attendees:
+            return True
         if self.startendeditor.changed or self.calendar_chooser.changed:
             return True
         if self.recurrenceeditor.changed:
@@ -440,6 +463,7 @@ class EventEditor(urwid.WidgetWrap):
         self.event.update_summary(get_wrapped_text(self.summary))
         self.event.update_description(get_wrapped_text(self.description))
         self.event.update_location(get_wrapped_text(self.location))
+        self.event.update_attendees(get_wrapped_text(self.attendees).split(','))
         self.event.update_categories(get_wrapped_text(self.categories).split(','))
         self.event.update_url(get_wrapped_text(self.url))
 
@@ -500,7 +524,7 @@ class EventEditor(urwid.WidgetWrap):
             self.event.increment_sequence()
             if self.event.etag is None:  # has not been saved before
                 self.event.calendar = self.calendar_chooser.active['name']
-                self.collection.new(self.event)
+                self.collection.insert(self.event)
             elif self.calendar_chooser.changed:
                 self.collection.change_collection(
                     self.event,
@@ -721,7 +745,7 @@ class RecurrenceEditor(urwid.WidgetWrap):
         return self._rrule != self.rrule()  # TODO do this properly
 
     def rrule(self):
-        rrule = dict()
+        rrule = {}
         rrule['freq'] = [self.recurrence_choice.active]
         interval = int(self.interval_edit.get_edit_text())
         if interval != 1:
