@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -24,11 +24,13 @@
 Widgets that are specific to calendaring/khal should go into __init__.py or,
 if they are large, into their own files
 """
-from datetime import date, datetime, timedelta
+import datetime as dt
 import re
 
 import urwid
 
+from .calendarwidget import CalendarWidget  # noqa
+
 
 class DateConversionError(Exception):
     pass
@@ -73,6 +75,7 @@ def goto_end_of_line(text):
 
 class ExtendedEdit(urwid.Edit):
     """A text editing widget supporting some more editing commands"""
+
     def keypress(self, size, key):
         if key == 'ctrl w':
             self._delete_word()
@@ -85,7 +88,7 @@ class ExtendedEdit(urwid.Edit):
         elif key == 'ctrl e':
             self._goto_end_of_line()
         else:
-            return super(ExtendedEdit, self).keypress(size, key)
+            return super().keypress(size, key)
 
     def _delete_word(self):
         """delete word before cursor"""
@@ -143,7 +146,7 @@ class DateTimeWidget(ExtendedEdit):
                 pass
             else:
                 self.on_date_change(new_date)
-        return super(DateTimeWidget, self).keypress(size, key)
+        return super().keypress(size, key)
 
     def increase(self):
         """call to increase the datefield by self.timedelta"""
@@ -171,12 +174,12 @@ class DateTimeWidget(ExtendedEdit):
 
 
 class DateWidget(DateTimeWidget):
-    dtype = date
-    timedelta = timedelta(days=1)
+    dtype = dt.date
+    timedelta = dt.timedelta(days=1)
 
     def _get_current_value(self):
         try:
-            new_date = datetime.strptime(self.get_edit_text(), self.dateformat).date()
+            new_date = dt.datetime.strptime(self.get_edit_text(), self.dateformat).date()
         except ValueError:
             raise DateConversionError
         else:
@@ -184,12 +187,12 @@ class DateWidget(DateTimeWidget):
 
 
 class TimeWidget(DateTimeWidget):
-    dtype = datetime
-    timedelta = timedelta(minutes=15)
+    dtype = dt.datetime
+    timedelta = dt.timedelta(minutes=15)
 
     def _get_current_value(self):
         try:
-            new_datetime = datetime.strptime(self.get_edit_text(), self.dateformat)
+            new_datetime = dt.datetime.strptime(self.get_edit_text(), self.dateformat)
         except ValueError:
             raise DateConversionError
         else:
@@ -252,7 +255,7 @@ class ChoiceList(urwid.WidgetWrap):
 
         pile = NPile(buttons, outermost=True)
         num = [num for num, elem in enumerate(parent.choices) if elem == parent.active][0]
-        pile.set_focus(num)
+        pile.focus_position = num
         fill = urwid.Filler(pile)
         urwid.WidgetWrap.__init__(self, urwid.AttrMap(fill, 'popupbg'))
 
@@ -262,31 +265,33 @@ class ChoiceList(urwid.WidgetWrap):
         self._emit("close")
 
 
-class SupportsNext(object):
+class SupportsNext:
     """classes inheriting from SupportsNext must implement the following methods:
     _select_first_selectable
     _select_last_selectable
     """
+
     def __init__(self, *args, **kwargs):
         self.outermost = kwargs.get('outermost', False)
         if 'outermost' in kwargs:
             kwargs.pop('outermost')
-        super(SupportsNext, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 class NextMixin(SupportsNext):
     """Implements SupportsNext for urwid.Pile and urwid.Columns"""
+
     def _select_first_selectable(self):
         """select our first selectable item (recursivly if that item SupportsNext)"""
         i = self._first_selectable()
-        self.set_focus(i)
+        self.focus_position = i
         if isinstance(self.contents[i][0], SupportsNext):
             self.contents[i][0]._select_first_selectable()
 
     def _select_last_selectable(self):
         """select our last selectable item (recursivly if that item SupportsNext)"""
         i = self._last_selectable()
-        self.set_focus(i)
+        self.focus_position = i
         if isinstance(self._contents[i][0], SupportsNext):
             self.contents[i][0]._select_last_selectable()
 
@@ -305,7 +310,7 @@ class NextMixin(SupportsNext):
         return False
 
     def keypress(self, size, key):
-        key = super(NextMixin, self).keypress(size, key)
+        key = super().keypress(size, key)
 
         if key == 'tab':
             if self.outermost and self.focus_position == self._last_selectable():
@@ -313,7 +318,7 @@ class NextMixin(SupportsNext):
             else:
                 for i in range(self.focus_position + 1, len(self._contents)):
                     if self._contents[i][0].selectable():
-                        self.set_focus(i)
+                        self.focus_position = i
                         if isinstance(self._contents[i][0], SupportsNext):
                             self._contents[i][0]._select_first_selectable()
                         break
@@ -325,7 +330,7 @@ class NextMixin(SupportsNext):
             else:
                 for i in range(self.focus_position - 1, 0 - 1, -1):
                     if self._contents[i][0].selectable():
-                        self.set_focus(i)
+                        self.focus_position = i
                         if isinstance(self._contents[i][0], SupportsNext):
                             self._contents[i][0]._select_last_selectable()
                         break
@@ -347,14 +352,14 @@ class NListBox(SupportsNext, urwid.ListBox):
     def _select_first_selectable(self):
         """select our first selectable item (recursivly if that item SupportsNext)"""
         i = self._first_selectable()
-        self.set_focus(i)
+        self.focus_position = i
         if isinstance(self.body[i], SupportsNext):
             self.body[i]._select_first_selectable()
 
     def _select_last_selectable(self):
         """select our last selectable item (recursivly if that item SupportsNext)"""
         i = self._last_selectable()
-        self.set_focus(i)
+        self.focus_position = i
         if isinstance(self.body[i], SupportsNext):
             self.body[i]._select_last_selectable()
 
@@ -392,7 +397,7 @@ class ValidatedEdit(urwid.WidgetWrap):
     def __init__(self, *args, EditWidget=ExtendedEdit, validate=False, **kwargs):
         assert validate
         self._validate_func = validate
-        self._original_widget = urwid.AttrMap(EditWidget(*args, **kwargs), 'edit', 'editf')
+        self._original_widget = urwid.AttrMap(EditWidget(*args, **kwargs), 'edit', 'edit focused')
         super().__init__(self._original_widget)
 
     @property
@@ -495,7 +500,7 @@ class DurationWidget(urwid.WidgetWrap):
         urwid.WidgetWrap.__init__(self, self.columns)
 
     def get_timedelta(self):
-        return timedelta(
+        return dt.timedelta(
             seconds=int(self.seconds_edit.get_edit_text()) +
             int(self.minutes_edit.get_edit_text()) * 60 +
             int(self.hours_edit.get_edit_text()) * 60 * 60 +
@@ -515,7 +520,8 @@ class AlarmsEditor(urwid.WidgetWrap):
                 duration = -1 * duration
 
             self.duration = DurationWidget(duration)
-            self.description = ExtendedEdit(edit_text=description)
+            self.description = ExtendedEdit(
+                edit_text=description if description is not None else "")
             self.direction = Choice(
                 ['before', 'after'], active=direction, overlay_width=10)
             self.columns = NColumns([
@@ -549,7 +555,7 @@ class AlarmsEditor(urwid.WidgetWrap):
     def add_alarm(self, button):
         self.pile.contents.insert(
             len(self.pile.contents) - 1,
-            (self.AlarmEditor((timedelta(0), self.event.summary), self.remove_alarm),
+            (self.AlarmEditor((dt.timedelta(0), self.event.summary), self.remove_alarm),
              ('weight', 1)))
 
     def remove_alarm(self, button, editor):
