@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2021 khal contributors
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -23,28 +23,32 @@
 import datetime as dt
 import logging
 import os
+import re
 import textwrap
 from collections import OrderedDict, defaultdict
 from shutil import get_terminal_size
+from typing import List, Optional
 
 import pytz
 from click import confirm, echo, prompt, style
-from khal import (__productname__, __version__, calendar_display,
-                  parse_datetime, utils)
-from khal.exceptions import FatalError, DateTimeParseError
+
+from khal import __productname__, __version__, calendar_display, parse_datetime, utils
+from khal.custom_types import EventCreationTypes, LocaleConfiguration
+from khal.exceptions import DateTimeParseError, FatalError
+from khal.khalendar import CalendarCollection
 from khal.khalendar.event import Event
 from khal.khalendar.exceptions import DuplicateUid, ReadOnlyCalendarError
 
 from .exceptions import ConfigurationError
-from .icalendar import (cal_from_ics, new_event as new_vevent, split_ics,
-                        sort_key as sort_vevent_key)
+from .icalendar import cal_from_ics, split_ics
+from .icalendar import sort_key as sort_vevent_key
 from .khalendar.vdir import Item
 from .terminal import merge_columns
 
 logger = logging.getLogger('khal')
 
 
-def format_day(day, format_string, locale, attributes=None):
+def format_day(day, format_string: str, locale, attributes=None):
     if attributes is None:
         attributes = {}
 
@@ -74,6 +78,7 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
              hmethod='fg',
              default_color='',
              multiple='',
+             multiple_on_overflow=False,
              color='',
              highlight_event_days=0,
              full=False,
@@ -104,16 +109,20 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
         width=rwidth,
         env=env,
     )
+    if not event_column:
+        event_column = [style('No events', bold=True)]
+    month_count = (end.year - start.year) * 12 + end.month - start.month + 1
     calendar_column = calendar_display.vertical_month(
         month=start.month,
         year=start.year,
-        count=max(3, (end.year - start.year) * 12 + end.month - start.month + 1),
+        count=max(conf['view']['min_calendar_display'], month_count),
         firstweekday=firstweekday, weeknumber=weeknumber,
         monthdisplay=monthdisplay,
         collection=collection,
         hmethod=hmethod,
         default_color=default_color,
         multiple=multiple,
+        multiple_on_overflow=multiple_on_overflow,
         color=color,
         highlight_event_days=highlight_event_days,
         locale=locale,
@@ -121,18 +130,19 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
     return merge_columns(calendar_column, event_column, width=lwidth)
 
 
-def start_end_from_daterange(daterange, locale,
-                             default_timedelta_date=dt.timedelta(days=1),
-                             default_timedelta_datetime=dt.timedelta(hours=1)):
+def start_end_from_daterange(
+    daterange: List[str],
+    locale: LocaleConfiguration,
+    default_timedelta_date: dt.timedelta=dt.timedelta(days=1),
+    default_timedelta_datetime: dt.timedelta=dt.timedelta(hours=1),
+):
     """
     convert a string description of a daterange into start and end datetime
 
     if no description is given, return (today, today + default_timedelta_date)
 
     :param daterange: an iterable of strings that describes `daterange`
-    :type daterange: tuple
     :param locale: locale settings
-    :type locale: dict
     """
     if not daterange:
         start = dt.datetime(*dt.date.today().timetuple()[:3])
@@ -146,26 +156,29 @@ def start_end_from_daterange(daterange, locale,
 
 
 def get_events_between(
-        collection, locale, start, end, agenda_format=None, notstarted=False,
-        env=None, width=None, seen=None, original_start=None):
+    collection: CalendarCollection,
+    locale: dict,
+    start: dt.datetime,
+    end: dt.datetime,
+    agenda_format: str,
+    notstarted: bool,
+    env: dict,
+    width,
+    seen,
+    original_start: dt.datetime,
+) -> List[str]:
     """returns a list of events scheduled between start and end. Start and end
     are strings or datetimes (of some kind).
 
     :param collection:
-    :type collection: khalendar.CalendarCollection
     :param start: the start datetime
     :param end: the end datetime
     :param agenda_format: a format string that can be used in python string formatting
-    :type  agenda_format: str
     :param env: a collection of "static" values like calendar names and color
-    :type env: dict
     :param nostarted: True if each event should start after start (instead of
     be active between start and end)
-    :type nostarted: bool
     :param original_start: start datetime to compare against of notstarted is set
-    :type original_start: datetime.datetime
     :returns: a list to be printed as the agenda for the given days
-    :rtype: list(str)
     """
     assert not (notstarted and not original_start)
 
@@ -209,11 +222,22 @@ def get_events_between(
     return event_list
 
 
-def khal_list(collection, daterange=None, conf=None, agenda_format=None,
-              day_format=None, once=False, notstarted=False, width=False,
-              env=None, datepoint=None):
-    assert daterange is not None or datepoint is not None
+def khal_list(
+    collection,
+    daterange: Optional[List[str]]=None,
+    conf: Optional[dict] = None,
+    agenda_format=None,
+    day_format: Optional[str]=None,
+    once=False,
+    notstarted: bool = False,
+    width: bool = False,
+    env=None,
+    datepoint=None,
+):
     """returns a list of all events in `daterange`"""
+    assert daterange is not None or datepoint is not None
+    assert conf is not None
+
     # because empty strings are also Falsish
     if agenda_format is None:
         agenda_format = conf['view']['agenda_event_format']
@@ -232,11 +256,13 @@ def khal_list(collection, daterange=None, conf=None, agenda_format=None,
         if not datepoint:
             datepoint = ['now']
         try:
+            # hand over a copy of the `datepoint` so error reporting works
+            # (we pop from that list in guessdatetimefstr())
             start, allday = parse_datetime.guessdatetimefstr(
-                datepoint, conf['locale'], dt.date.today(),
+                list(datepoint), conf['locale'], dt.date.today(),
             )
-        except ValueError:
-            raise FatalError('Invalid value of `{' '.join(datepoint)}` for a datetime')
+        except (ValueError, IndexError):
+            raise FatalError(f"Invalid value of {' '.join(datepoint)} for a datetime")
         if allday:
             logger.debug(f'Got date {start}')
             raise FatalError('Please supply a datetime, not a date.')
@@ -248,7 +274,7 @@ def khal_list(collection, daterange=None, conf=None, agenda_format=None,
             )
         logger.debug(f'Getting all events between {start} and {end}')
 
-    event_column = []
+    event_column: List[str] = []
     once = set() if once else None
     if env is None:
         env = {}
@@ -273,20 +299,19 @@ def khal_list(collection, daterange=None, conf=None, agenda_format=None,
         event_column.extend(current_events)
         start = dt.datetime(*start.date().timetuple()[:3]) + dt.timedelta(days=1)
 
-    if event_column == []:
-        event_column = [style('No events', bold=True)]
     return event_column
 
 
 def new_interactive(collection, calendar_name, conf, info, location=None,
                     categories=None, repeat=None, until=None, alarms=None,
                     format=None, env=None, url=None):
+    info: EventCreationTypes
     try:
         info = parse_datetime.eventinfofstr(
             info, conf['locale'],
             conf['default']['default_event_duration'],
             conf['default']['default_dayevent_duration'],
-            adjust_reasonably=True, localize=False,
+            adjust_reasonably=True,
         )
     except DateTimeParseError:
         info = {}
@@ -330,12 +355,23 @@ def new_interactive(collection, calendar_name, conf, info, location=None,
     if info['description'] == 'None':
         info['description'] = ''
 
-    event = new_from_args(
-        collection, calendar_name, conf, format=format, env=env,
-        location=location, categories=categories,
-        repeat=repeat, until=until, alarms=alarms, url=url,
-        **info)
+    info.update({
+        'location': location,
+        'categories': categories,
+        'repeat': repeat,
+        'until': until,
+        'alarms': alarms,
+        'url': url,
+    })
 
+    event = new_from_dict(
+        info,
+        collection,
+        conf,
+        format=format,
+        env=env,
+        calendar_name=calendar_name,
+    )
     echo("event saved")
 
     term_width, _ = get_terminal_size()
@@ -350,38 +386,44 @@ def new_from_string(collection, calendar_name, conf, info, location=None,
         info, conf['locale'],
         conf['default']['default_event_duration'],
         conf['default']['default_dayevent_duration'],
-        adjust_reasonably=True, localize=False
-    )
-    new_from_args(
-        collection, calendar_name, conf, format=format, env=env,
-        location=location, categories=categories, repeat=repeat,
-        until=until, alarms=alarms, url=url, **info
+        adjust_reasonably=True,
     )
+    info.update({
+        'location': location,
+        'categories': categories,
+        'repeat': repeat,
+        'until': until,
+        'alarms': alarms,
+        'url': url,
+    })
+    new_from_dict(info, collection, conf=conf, format=format, env=env, calendar_name=calendar_name)
 
 
-def new_from_args(collection, calendar_name, conf, dtstart=None, dtend=None,
-                  summary=None, description=None, allday=None, location=None,
-                  categories=None, repeat=None, until=None, alarms=None,
-                  timezone=None, url=None, format=None, env=None):
-    """Create a new event from arguments and add to vdirs"""
-    if isinstance(categories, str):
-        categories = [category.strip() for category in categories.split(',')]
+def new_from_dict(
+    event_args: EventCreationTypes,
+    collection: CalendarCollection,
+    conf,
+    calendar_name: Optional[str]=None,
+    format=None,
+    env=None,
+) -> Event:
+    """Create a new event from arguments and save in vdirs
+
+    This is a wrapper around CalendarCollection.create_event_from_dict()
+    """
+    if isinstance(event_args['categories'], str):
+        event_args['categories'] = [event_args['categories'].strip()
+                                    for category in event_args['categories'].split(',')]
     try:
-        event = new_vevent(
-            locale=conf['locale'], location=location, categories=categories,
-            repeat=repeat, until=until, alarms=alarms, dtstart=dtstart,
-            dtend=dtend, summary=summary, description=description, timezone=timezone, url=url,
-        )
+        event = collection.create_event_from_dict(event_args, calendar_name=calendar_name)
     except ValueError as error:
         raise FatalError(error)
-    event = Event.fromVEvents(
-        [event], calendar=calendar_name, locale=conf['locale'])
 
     try:
-        collection.new(event)
+        collection.insert(event)
     except ReadOnlyCalendarError:
         raise FatalError(
-            f'ERROR: Cannot modify calendar "{calendar_name}" as it is read-only'
+            f'ERROR: Cannot modify calendar `{calendar_name}` as it is read-only'
         )
 
     if conf['default']['print_new'] == 'event':
@@ -389,10 +431,8 @@ def new_from_args(collection, calendar_name, conf, dtstart=None, dtend=None,
             format = conf['view']['event_format']
         echo(event.format(format, dt.datetime.now(), env=env))
     elif conf['default']['print_new'] == 'path':
-        path = os.path.join(
-            collection._calendars[event.calendar]['path'],
-            event.href
-        )
+        assert event.href
+        path = os.path.join(collection._calendars[event.calendar]['path'], event.href)
         echo(path)
     return event
 
@@ -429,6 +469,11 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
     options["alarm"] = {"short": "a"}
     options["Delete"] = {"short": "D"}
     options["url"] = {"short": "u", "attr": "url", "none": True}
+    # some output contains ansi escape sequences (probably only resets)
+    # if hitting enter, the output (including the escape sequence) gets parsed
+    # and fails the parsing. Therefore we remove ansi escape sequences before
+    # parsing.
+    ansi = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
 
     now = dt.datetime.now()
 
@@ -452,7 +497,7 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
             current = event.format("{start} {end}", relative_to=now)
             value = prompt("datetime range", default=current)
             try:
-                start, end, allday = parse_datetime.guessrangefstr(value, locale)
+                start, end, allday = parse_datetime.guessrangefstr(ansi.sub('', value), locale)
                 event.update_start_end(start, end)
                 edited = True
             except:  # noqa
@@ -470,7 +515,7 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
                 until = prompt('until (or "None")', until)
                 if until == 'None':
                     until = None
-                rrule = parse_datetime.rrulefstr(freq, until, locale)
+                rrule = parse_datetime.rrulefstr(freq, until, locale, event.start.tzinfo)
                 event.update_rrule(rrule)
             edited = True
         elif choice == "alarm":
@@ -564,7 +609,10 @@ def import_ics(collection, conf, ics, batch=False, random_uid=False, format=None
     """
     if format is None:
         format = conf['view']['event_format']
-    vevents = split_ics(ics, random_uid, conf['locale']['default_timezone'])
+    try:
+        vevents = split_ics(ics, random_uid, conf['locale']['default_timezone'])
+    except Exception as error:
+        raise FatalError(error)
     for vevent in vevents:
         import_event(vevent, collection, conf['locale'], batch, format, env)
 
@@ -614,7 +662,7 @@ def import_event(vevent, collection, locale, batch, format=None, env=None):
 
     if batch or confirm(f"Do you want to import this event into `{calendar_name}`?"):
         try:
-            collection.new(Item(vevent), collection=calendar_name)
+            collection.insert(Item(vevent), collection=calendar_name)
         except DuplicateUid:
             if batch or confirm(
                     "An event with the same UID already exists. Do you want to update it?"):
