@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2021 khal contributors
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -24,22 +24,24 @@ The SQLite backend implementation.
 
 import contextlib
 import datetime as dt
-from enum import IntEnum
 import logging
 import sqlite3
+from enum import IntEnum
 from os import makedirs, path
-from typing import Any, Dict, Iterable, List, Optional, Tuple
+from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple, Union
 
 import icalendar
+import icalendar.cal
 import pytz
 from dateutil import parser
 
 from .. import utils
-from ..icalendar import (cal_from_ics, assert_only_one_uid,
-                         expand as expand_vevent, sanitize as sanitize_vevent,
-                         sort_key as sort_vevent_key)
-from .exceptions import (CouldNotCreateDbDir, OutdatedDbVersionError,
-                         UpdateFailed, NonUniqueUID)
+from ..custom_types import EventTuple
+from ..icalendar import assert_only_one_uid, cal_from_ics
+from ..icalendar import expand as expand_vevent
+from ..icalendar import sanitize as sanitize_vevent
+from ..icalendar import sort_key as sort_vevent_key
+from .exceptions import CouldNotCreateDbDir, NonUniqueUID, OutdatedDbVersionError, UpdateFailed
 
 logger = logging.getLogger('khal')
 
@@ -76,11 +78,11 @@ class SQLiteDb:
                  locale: Dict[str, str],
                  ) -> None:
         assert db_path is not None
-        self.calendars = list(calendars)
+        self.calendars: List[str] = list(calendars)
         self.db_path = path.expanduser(db_path)
         self._create_dbdir()
         self.locale = locale
-        self._at_once = False
+        self._at_once: bool = False
         self.conn = sqlite3.connect(self.db_path)
         self.cursor = self.conn.cursor()
         self._create_default_tables()
@@ -88,7 +90,7 @@ class SQLiteDb:
         self._check_table_version()
 
     @contextlib.contextmanager
-    def at_once(self):
+    def at_once(self) -> Iterator['SQLiteDb']:
         assert not self._at_once
         self._at_once = True
         try:
@@ -195,7 +197,12 @@ class SQLiteDb:
             self.conn.commit()
         return result
 
-    def update(self, vevent_str: str, href: str, etag: str='', calendar: str=None) -> None:
+    def update(self,
+               vevent_str: str,
+               href: str,
+               etag: str='',
+               calendar: Optional[str]=None,
+               ) -> None:
         """insert a new or update an existing event into the db
 
         This is mostly a wrapper around two SQL statements, doing some cleanup
@@ -243,7 +250,7 @@ class SQLiteDb:
         self.sql_ex(sql_s, stuple)
 
     def update_vcf_dates(self, vevent_str: str, href: str, etag: str='',
-                         calendar: str=None) -> None:
+                         calendar: Optional[str]=None) -> None:
         """insert events from a vcard into the db
 
         This is will parse BDAY, ANNIVERSARY, X-ANNIVERSARY and X-ABDATE fields.
@@ -268,6 +275,7 @@ class SQLiteDb:
         for key in vcard.keys():
             if key in ['BDAY', 'X-ANNIVERSARY', 'ANNIVERSARY'] or key.endswith('X-ABDATE'):
                 date = vcard[key]
+                uuid = vcard.get('UID')
                 if isinstance(date, list):
                     logger.warning(
                         f'Vcard {href} in collection {calendar} has more than one '
@@ -317,7 +325,12 @@ class SQLiteDb:
                 sql_s = ('INSERT INTO events (item, etag, href, calendar)'
                          ' VALUES (?, ?, ?, ?);')
                 stuple = (vevent_str, etag, href + key, calendar)
-                self.sql_ex(sql_s, stuple)
+                try:
+                    self.sql_ex(sql_s, stuple)
+                except sqlite3.IntegrityError as error:
+                    raise UpdateFailed('Database integrity error creating birthday event '
+                                       'on {} for contact {} (UID: {}): '
+                                       '{}'.format(date, name, uuid, error))
 
     def _update_impl(self, vevent: icalendar.cal.Event, href: str, calendar: str) -> None:
         """insert `vevent` into the database
@@ -389,7 +402,7 @@ class SQLiteDb:
                 stuple_n = (dbstart, dbend, href, ref, dtype, rec_inst, calendar)
                 self.sql_ex(recs_sql_s, stuple_n)
 
-    def get_ctag(self, calendar=str) -> Optional[str]:
+    def get_ctag(self, calendar: str) -> Optional[str]:
         stuple = (calendar, )
         sql_s = 'SELECT ctag FROM calendars WHERE calendar = ?;'
         try:
@@ -398,7 +411,7 @@ class SQLiteDb:
         except IndexError:
             return None
 
-    def set_ctag(self, ctag: str, calendar: str):
+    def set_ctag(self, ctag: str, calendar: str) -> None:
         stuple = (ctag, calendar, )
         sql_s = 'UPDATE calendars SET ctag = ? WHERE calendar = ?;'
         self.sql_ex(sql_s, stuple)
@@ -407,9 +420,7 @@ class SQLiteDb:
     def get_etag(self, href: str, calendar: str) -> Optional[str]:
         """get etag for href
 
-        type href: str()
         return: etag
-        rtype: str()
         """
         sql_s = 'SELECT etag FROM events WHERE href = ? AND calendar = ?;'
         try:
@@ -418,22 +429,21 @@ class SQLiteDb:
         except IndexError:
             return None
 
-    def delete(self, href: str, etag: Any=None, calendar: str=None):
+    def delete(self, href: str, etag: Any=None, calendar: str='') -> None:
         """
         removes the event from the db,
 
         :param etag: only there for compatibility with vdirsyncer's Storage,
                      we always delete
-        :returns: None
         """
-        assert calendar is not None
+        assert calendar != ''
         for table in ['recs_loc', 'recs_float']:
             sql_s = f'DELETE FROM {table} WHERE href = ? AND calendar = ?;'
             self.sql_ex(sql_s, (href, calendar))
         sql_s = 'DELETE FROM events WHERE href = ? AND calendar = ?;'
         self.sql_ex(sql_s, (href, calendar))
 
-    def deletelike(self, href: str, etag: Any=None, calendar: str=None):
+    def deletelike(self, href: str, etag: Any=None, calendar: str='') -> None:
         """
         removes events from the db that match an SQL 'like' statement,
 
@@ -441,16 +451,15 @@ class SQLiteDb:
                      like '%'
         :param etag: only there for compatibility with vdirsyncer's Storage,
                      we always delete
-        :returns: None
         """
-        assert calendar is not None
+        assert calendar != ''
         for table in ['recs_loc', 'recs_float']:
             sql_s = f'DELETE FROM {table} WHERE href LIKE ? AND calendar = ?;'
             self.sql_ex(sql_s, (href, calendar))
         sql_s = 'DELETE FROM events WHERE href LIKE ? AND calendar = ?;'
         self.sql_ex(sql_s, (href, calendar))
 
-    def list(self, calendar):
+    def list(self, calendar: str) -> List[Tuple[str, str]]:
         """ list all events in `calendar`
 
         used for testing
@@ -479,18 +488,11 @@ class SQLiteDb:
         for calendar in result:
             yield calendar[0]  # result is always an iterable, even if getting only one item
 
-    def get_localized(self, start, end) \
-            -> Iterable[Tuple[str, str, dt.datetime, dt.datetime, str, str, str]]:
-        """returns
-        :type start: datetime.datetime
-        :type end: datetime.datetime
-        :param minimal: if set, we do not return an event but a minimal stand in
-        :type minimal: bool
-        """
+    def get_localized(self, start: dt.datetime, end: dt.datetime) -> Iterable[EventTuple]:
         assert start.tzinfo is not None
         assert end.tzinfo is not None
-        start = utils.to_unix_time(start)
-        end = utils.to_unix_time(end)
+        start_timestamp = utils.to_unix_time(start)
+        end_timestamp = utils.to_unix_time(end)
         sql_s = (
             'SELECT item, recs_loc.href, dtstart, dtend, ref, etag, dtype, events.calendar '
             'FROM recs_loc JOIN events ON '
@@ -498,13 +500,22 @@ class SQLiteDb:
             'recs_loc.calendar = events.calendar WHERE '
             '(dtstart >= ? AND dtstart <= ? OR '
             'dtend > ? AND dtend <= ? OR '
-            'dtstart <= ? AND dtend >= ?) AND events.calendar in ({0}) '
+            'dtstart <= ? AND dtend >= ?) AND '
+            # insert as many "?" as we have configured calendars
+            f'events.calendar in ({",".join("?" * len(self.calendars))}) '
             'ORDER BY dtstart')
-        stuple = tuple([start, end, start, end, start, end] + list(self.calendars))
-        result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
-        for item, href, start, end, ref, etag, _dtype, calendar in result:
-            start = pytz.UTC.localize(dt.datetime.utcfromtimestamp(start))
-            end = pytz.UTC.localize(dt.datetime.utcfromtimestamp(end))
+        stuple = (
+            start_timestamp,
+            end_timestamp,
+            start_timestamp,
+            end_timestamp,
+            start_timestamp,
+            end_timestamp,
+        ) + tuple(self.calendars)
+        result = self.sql_ex(sql_s, stuple)
+        for item, href, start_timestamp, end_timestamp, ref, etag, _dtype, calendar in result:
+            start = pytz.UTC.localize(dt.datetime.utcfromtimestamp(start_timestamp))
+            end = pytz.UTC.localize(dt.datetime.utcfromtimestamp(end_timestamp))
             yield item, href, start, end, ref, etag, calendar
 
     def get_floating_calendars(self, start: dt.datetime, end: dt.datetime) -> Iterable[str]:
@@ -527,15 +538,13 @@ class SQLiteDb:
         for calendar in result:
             yield calendar[0]
 
-    def get_floating(self, start, end) \
-            -> Iterable[Tuple[str, str, dt.datetime, dt.datetime, str, str, str]]:
-        """return floating events between `start` and `end`
-
-        :type start: datetime.datetime
-        :type end: datetime.datetime
-        """
+    def get_floating(self, start: dt.datetime, end: dt.datetime) -> Iterable[EventTuple]:
+        """return floating events between `start` and `end`"""
         assert start.tzinfo is None
         assert end.tzinfo is None
+        start_dt: Union[dt.datetime, dt.date]
+        end_dt: Union[dt.datetime, dt.date]
+
         start_u = utils.to_unix_time(start)
         end_u = utils.to_unix_time(end)
         sql_s = (
@@ -550,13 +559,13 @@ class SQLiteDb:
         stuple = tuple(
             [start_u, end_u, start_u, end_u, start_u, end_u] + list(self.calendars))  # type: ignore
         result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
-        for item, href, start, end, ref, etag, dtype, calendar in result:
-            start = dt.datetime.utcfromtimestamp(start)
-            end = dt.datetime.utcfromtimestamp(end)
+        for item, href, start_s, end_s, ref, etag, dtype, calendar in result:
+            start_dt = dt.datetime.utcfromtimestamp(start_s)
+            end_dt = dt.datetime.utcfromtimestamp(end_s)
             if dtype == EventType.DATE:
-                start = start.date()
-                end = end.date()
-            yield item, href, start, end, ref, etag, calendar
+                start_dt = start_dt.date()
+                end_dt = end_dt.date()
+            yield item, href, start_dt, end_dt, ref, etag, calendar
 
     def get(self, href: str, calendar: str) -> str:
         """returns the ical string matching href and calendar"""
@@ -565,8 +574,15 @@ class SQLiteDb:
         item, etag = self.sql_ex(sql_s, (href, calendar))[0]
         return item
 
+    def get_with_etag(self, href: str, calendar: str) -> Tuple[str, str]:
+        """returns the ical string and its etag matching href and calendar"""
+        assert calendar is not None
+        sql_s = 'SELECT item, etag FROM events WHERE href = ? AND calendar = ?;'
+        item, etag = self.sql_ex(sql_s, (href, calendar))[0]
+        return item, etag
+
     def search(self, search_string: str) \
-            -> Iterable[Tuple[str, str, dt.datetime, dt.datetime, str, str, str]]:
+            -> Iterable[Tuple[str, str, dt.date, dt.date, str, str, str]]:
         """search for events matching `search_string`"""
         sql_s = (
             'SELECT item, recs_loc.href, dtstart, dtend, ref, etag, dtype, events.calendar '
@@ -603,7 +619,7 @@ class SQLiteDb:
             yield item, href, start, end, ref, etag, calendar
 
 
-def check_support(vevent: icalendar.cal.Event, href: str, calendar: str):
+def check_support(vevent: icalendar.cal.Event, href: str, calendar: str) -> None:
     """test if all icalendar features used in this event are supported,
     raise `UpdateFailed` otherwise.
     :param vevent: event to test
@@ -628,7 +644,7 @@ def check_support(vevent: icalendar.cal.Event, href: str, calendar: str):
         )
 
 
-def check_for_errors(component: icalendar.cal.Component, calendar: str, href: str):
+def check_for_errors(component: icalendar.cal.Component, calendar: str, href: str) -> None:
     """checking if component.errors exists, is not empty and if so warn the user"""
     if hasattr(component, 'errors') and component.errors:
         logger.error(
@@ -643,7 +659,7 @@ def calc_shift_deltas(vevent: icalendar.Event) -> Tuple[dt.timedelta, dt.timedel
     """calculate an event's duration and by how much its start time has shifted
     versus its recurrence-id time
 
-    :param event: an event with an RECURRENCE-ID property
+    :param event: an event with a RECURRENCE-ID property
     """
     assert isinstance(vevent, icalendar.Event)  # REMOVE ME
     start_shift = vevent['DTSTART'].dt - vevent['RECURRENCE-ID'].dt
