@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -25,26 +25,30 @@ helper functions."""
 import datetime as dt
 import logging
 import os
-from collections import defaultdict
+from typing import Dict, List, Optional, Type, Union
 
 import icalendar
+import icalendar.cal
+import icalendar.prop
 import pytz
 from click import style
+from pytz.tzinfo import StaticTzInfo
 
+from ..custom_types import LocaleConfiguration
 from ..exceptions import FatalError
-from ..terminal import get_color
-from ..utils import (cal_from_ics, delete_instance, generate_random_uid,
-                     invalid_timezone, is_aware, to_naive_utc, to_unix_time)
+from ..icalendar import cal_from_ics, delete_instance, invalid_timezone
 from ..parse_datetime import timedelta2str
+from ..terminal import get_color
+from ..utils import generate_random_uid, is_aware, to_naive_utc, to_unix_time
 
 logger = logging.getLogger('khal')
 
 
-class Event(object):
+class Event:
     """base Event class for representing a *recurring instance* of an Event
 
     (in case of non-recurring events this distinction is irrelevant)
-    We keep a copy of the start and end time around, because for recurring
+    We keep a copy of this instances start and end time around, because for recurring
     events it might be costly to expand the recursion rules
 
     important distinction for AllDayEvents:
@@ -52,28 +56,38 @@ class Event(object):
         only one day will have the same start and end date (even though the
         icalendar standard would have the end date be one day later)
     """
-    allday = False
+    allday: bool = False
 
-    def __init__(self, vevents, ref=None, **kwargs):
+    def __init__(self,
+                 vevents: Dict[str, icalendar.Event],
+                 locale: LocaleConfiguration,
+                 ref: Optional[str] = None,
+                 readonly: bool = False,
+                 href: Optional[str] = None,
+                 etag: Optional[str] = None,
+                 calendar: Optional[str] = None,
+                 color: Optional[str] = None,
+                 start: Optional[dt.datetime] = None,
+                 end: Optional[dt.datetime] = None,
+                 ):
         """
         :param start: start datetime of this event instance
-        :type start: datetime.date
-        :param end: end datetime of this event instance in unix time
-        :type end: datetime.date
+        :param end: end datetime of this event instance
         """
         if self.__class__.__name__ == 'Event':
             raise ValueError('do not initialize this class directly')
+        if ref is None:
+            raise ValueError('ref should not be None')
         self._vevents = vevents
-        self._locale = kwargs.pop('locale', None)
-        self.readonly = kwargs.pop('readonly', None)
-        self.href = kwargs.pop('href', None)
-        self.etag = kwargs.pop('etag', None)
-        self.calendar = kwargs.pop('calendar', None)
-        self.color = kwargs.pop('color', None)
         self.ref = ref
-
-        start = kwargs.pop('start', None)
-        end = kwargs.pop('end', None)
+        self._locale = locale
+        self.readonly = readonly
+        self.href = href
+        self.etag = etag
+        self.calendar = calendar if calendar else ''
+        self.color = color
+        self._start: dt.datetime
+        self._end: dt.datetime
 
         if start is None:
             self._start = self._vevents[self.ref]['DTSTART'].dt
@@ -87,18 +101,12 @@ class Event(object):
                     self._end = self._start + self._vevents[self.ref]['DURATION'].dt
                 except KeyError:
                     self._end = self._start + dt.timedelta(days=1)
-
         else:
             self._end = end
-        if kwargs:
-            raise TypeError('%s are invalid keyword arguments to this function' % kwargs.keys())
 
     @classmethod
-    def _get_type_from_vDDD(cls, start):
-        """
-        :type start: icalendar.prop.vDDDTypes
-        :type start: icalendar.prop.vDDDTypes
-        """
+    def _get_type_from_vDDD(cls, start: icalendar.prop.vDDDTypes) -> type:
+        """infere the type of the class from the START type of the event"""
         if not isinstance(start.dt, dt.datetime):
             return AllDayEvent
         if 'TZID' in start.params or start.dt.tzinfo is not None:
@@ -106,7 +114,7 @@ class Event(object):
         return FloatingEvent
 
     @classmethod
-    def _get_type_from_date(cls, start):
+    def _get_type_from_date(cls, start: dt.datetime) -> Type['Event']:
         if hasattr(start, 'tzinfo') and start.tzinfo is not None:
             cls = LocalizedEvent
         elif isinstance(start, dt.datetime):
@@ -116,13 +124,14 @@ class Event(object):
         return cls
 
     @classmethod
-    def fromVEvents(cls, events_list, ref=None, **kwargs):
-        """
-        :type events: list
-        """
+    def fromVEvents(cls,
+                    events_list: List[icalendar.Event],
+                    ref: Optional[str]=None,
+                    start: Optional[dt.datetime]=None,
+                    **kwargs) -> 'Event':
         assert isinstance(events_list, list)
 
-        vevents = dict()
+        vevents = {}
         for event in events_list:
             if 'RECURRENCE-ID' in event:
                 if invalid_timezone(event['RECURRENCE-ID']):
@@ -138,40 +147,61 @@ class Event(object):
         if ref is None:
             ref = 'PROTO' if ref in vevents.keys() else list(vevents.keys())[0]
         try:
-            if type(vevents[ref]['DTSTART'].dt) != type(vevents[ref]['DTEND'].dt):  # flake8: noqa
+            if type(vevents[ref]['DTSTART'].dt) != type(vevents[ref]['DTEND'].dt):  # noqa: E721
                 raise ValueError('DTSTART and DTEND should be of the same type (datetime or date)')
         except KeyError:
             pass
 
-        if kwargs.get('start'):
-            instcls = cls._get_type_from_date(kwargs.get('start'))
+        if start:
+            instcls = cls._get_type_from_date(start)
         else:
             instcls = cls._get_type_from_vDDD(vevents[ref]['DTSTART'])
-        return instcls(vevents, ref=ref, **kwargs)
+        return instcls(vevents, ref=ref, start=start, **kwargs)
 
     @classmethod
-    def fromString(cls, event_str, ref=None, **kwargs):
-        calendar_collection = cal_from_ics(event_str)
+    def fromString(cls, ics: str, ref=None, **kwargs) -> 'Event':
+        calendar_collection = cal_from_ics(ics)
         events = [item for item in calendar_collection.walk() if item.name == 'VEVENT']
         return cls.fromVEvents(events, ref, **kwargs)
 
-    def __lt__(self, other):
+    def __lt__(self, other: 'Event') -> bool:
         start = self.start_local
         other_start = other.start_local
         if isinstance(start, dt.date) and not isinstance(start, dt.datetime):
             start = dt.datetime.combine(start, dt.time.min)
 
         if isinstance(other_start, dt.date) and not isinstance(other_start, dt.datetime):
-            other_start = dt.datetime.combine(other_start, dt.time.max)
+            other_start = dt.datetime.combine(other_start, dt.time.min)
 
         start = start.replace(tzinfo=None)
         other_start = other_start.replace(tzinfo=None)
-        try:
-            return start <= other_start
-        except TypeError:
-            raise ValueError('Cannot compare events {} and {}'.format(start, other_start))
 
-    def update_start_end(self, start, end):
+        if start == other_start:
+            end = self.end_local
+            other_end = other.end_local
+            if isinstance(end, dt.date) and not isinstance(end, dt.datetime):
+                end = dt.datetime.combine(end, dt.time.min)
+
+            if isinstance(other_end, dt.date) and not isinstance(other_end, dt.datetime):
+                other_end = dt.datetime.combine(other_end, dt.time.min)
+
+            end = end.replace(tzinfo=None)
+            other_end = other_end.replace(tzinfo=None)
+
+            if end == other_end:
+                return self.summary < other.summary
+
+            try:
+                return end < other_end
+            except TypeError:
+                raise ValueError(f'Cannot compare events {end} and {other_end}')
+
+        try:
+            return start < other_start
+        except TypeError:
+            raise ValueError(f'Cannot compare events {start} and {other_start}')
+
+    def update_start_end(self, start: dt.datetime, end: dt.datetime) -> None:
         """update start and end time of this event
 
         calling this on a recurring event will lead to the proto instance
@@ -179,7 +209,7 @@ class Event(object):
 
         beware, this methods performs some open heart surgery
         """
-        if type(start) != type(end):  # flake8: noqa
+        if type(start) != type(end):
             raise ValueError('DTSTART and DTEND should be of the same type (datetime or date)')
         self.__class__ = self._get_type_from_date(start)
 
@@ -197,32 +227,41 @@ class Event(object):
             self._vevents[self.ref].add('DURATION', end - start)
 
     @property
-    def recurring(self):
-        return 'RRULE' in self._vevents[self.ref] or \
-            'RECURRENCE-ID' in self._vevents[self.ref] or \
-            'RDATE' in self._vevents[self.ref]
+    def recurring(self) -> bool:
+        try:
+            rval = 'RRULE' in self._vevents[self.ref] or \
+                'RECURRENCE-ID' in self._vevents[self.ref] or \
+                'RDATE' in self._vevents[self.ref]
+        except KeyError:
+            logger.fatal(
+                f"The event at {self.href} might be broken. You might want to "
+                "file an issue at https://github.com/pimutils/khal/issues"
+            )
+            raise
+        else:
+            return rval
 
     @property
-    def recurpattern(self):
+    def recurpattern(self) -> str:
         if 'RRULE' in self._vevents[self.ref]:
             return self._vevents[self.ref]['RRULE'].to_ical().decode('utf-8')
         else:
             return ''
 
     @property
-    def recurobject(self):
+    def recurobject(self) -> icalendar.vRecur:
         if 'RRULE' in self._vevents[self.ref]:
             return self._vevents[self.ref]['RRULE']
         else:
             return icalendar.vRecur()
 
-    def update_rrule(self, rrule):
+    def update_rrule(self, rrule: str) -> None:
         self._vevents['PROTO'].pop('RRULE')
         if rrule is not None:
             self._vevents['PROTO'].add('RRULE', rrule)
 
     @property
-    def recurrence_id(self):
+    def recurrence_id(self) -> Union[dt.datetime, str]:
         """return the "original" start date of this event (i.e. their recurrence-id)
         """
         if self.ref == 'PROTO':
@@ -230,7 +269,7 @@ class Event(object):
         else:
             return pytz.UTC.localize(dt.datetime.utcfromtimestamp(int(self.ref)))
 
-    def increment_sequence(self):
+    def increment_sequence(self) -> None:
         """update the SEQUENCE number, call before saving this event"""
         # TODO we might want to do this automatically in raw() everytime
         # the event has changed, this will f*ck up the tests though
@@ -240,76 +279,85 @@ class Event(object):
             self._vevents[self.ref]['SEQUENCE'] = 0
 
     @property
-    def symbol_strings(self):
+    def symbol_strings(self) -> Dict[str, str]:
         if self._locale['unicode_symbols']:
-            return dict(
-                recurring='\N{Clockwise gapped circle arrow}',
-                range='\N{Left right arrow}',
-                range_end='\N{Rightwards arrow to bar}',
-                range_start='\N{Rightwards arrow from bar}',
-                right_arrow='\N{Rightwards arrow}'
-            )
+            return {
+                'recurring': '\N{Clockwise gapped circle arrow}',
+                'alarming': '\N{Alarm clock}',
+                'range': '\N{Left right arrow}',
+                'range_end': '\N{Rightwards arrow to bar}',
+                'range_start': '\N{Rightwards arrow from bar}',
+                'right_arrow': '\N{Rightwards arrow}'
+            }
         else:
-            return dict(
-                recurring='(R)',
-                range='<->',
-                range_end='->|',
-                range_start='|->',
-                right_arrow='->'
-            )
+            return {
+                'recurring': '(R)',
+                'alarming': '(A)',
+                'range': '<->',
+                'range_end': '->|',
+                'range_start': '|->',
+                'right_arrow': '->'
+            }
 
     @property
-    def start_local(self):
+    def start_local(self) -> dt.datetime:
         """self.start() localized to local timezone"""
         return self.start
 
     @property
-    def end_local(self):
+    def end_local(self) -> dt.datetime:
         """self.end() localized to local timezone"""
         return self.end
 
     @property
-    def start(self):
+    def start(self) -> dt.datetime:
         """this should return the start date(time) as saved in the event"""
         return self._start
 
     @property
-    def end(self):
+    def end(self) -> dt.datetime:
         """this should return the end date(time) as saved in the event or
         implicitly defined by start and duration"""
         return self._end
 
     @property
-    def duration(self):
+    def duration(self) -> dt.timedelta:
         try:
             return self._vevents[self.ref]['DURATION'].dt
         except KeyError:
             return self.end - self.start
 
     @property
-    def uid(self):
+    def uid(self) -> str:
         return self._vevents[self.ref]['UID']
 
     @property
-    def organizer(self):
+    def organizer(self) -> str:
         if 'ORGANIZER' not in self._vevents[self.ref]:
             return ''
         organizer = self._vevents[self.ref]['ORGANIZER']
         cn = organizer.params.get('CN', '')
         email = organizer.split(':')[-1]
         if cn:
-            return '{} ({})'.format(cn, email)
+            return f'{cn} ({email})'
         else:
             return email
 
-    @staticmethod
-    def _create_calendar():
-        """
-        create the calendar
+    @property
+    def url(self) -> str:
+        if 'URL' not in self._vevents[self.ref]:
+            return ''
+        return self._vevents[self.ref]['URL']
 
-        :returns: calendar
-        :rtype: icalendar.Calendar()
-        """
+    def update_url(self, url: str) -> None:
+        if url:
+            self._vevents[self.ref]['URL'] = url
+        else:
+            self._vevents[self.ref].pop('URL')
+
+    @staticmethod
+    def _create_calendar() -> icalendar.Calendar:
+        """create the calendar"""
         calendar = icalendar.Calendar()
         calendar.add('version', '2.0')
         calendar.add(
@@ -318,13 +366,11 @@ class Event(object):
         return calendar
 
     @property
-    def raw(self):
-        """needed for vdirsyncer compatibility
-
-        return text
+    def raw(self) -> str:
+        """Creates a VCALENDAR containing VTIMEZONEs
         """
         calendar = self._create_calendar()
-        tzs = list()
+        tzs = []
         for vevent in self._vevents.values():
             if hasattr(vevent['DTSTART'].dt, 'tzinfo') and vevent['DTSTART'].dt.tzinfo is not None:
                 tzs.append(vevent['DTSTART'].dt.tzinfo)
@@ -343,7 +389,7 @@ class Event(object):
             calendar.add_component(vevent)
         return calendar.to_ical().decode('utf-8')
 
-    def export_ics(self, path):
+    def export_ics(self, path: str) -> None:
         """export event as ICS
         """
         export_path = os.path.expanduser(path)
@@ -351,12 +397,24 @@ class Event(object):
             fh.write(self.raw)
 
     @property
-    def summary(self):
-        bday = self._vevents[self.ref].get('x-birthday', None)
-        if bday:
-            number = self.start_local.year - int(bday[:4])
+    def summary(self) -> str:
+        description = None
+        date = self._vevents[self.ref].get('x-birthday', None)
+        if date:
+            description = 'birthday'
+        else:
+            date = self._vevents[self.ref].get('x-anniversary', None)
+            if date:
+                description = 'anniversary'
+            else:
+                date = self._vevents[self.ref].get('x-abdate', None)
+                if date:
+                    description = self._vevents[self.ref].get('x-ablabel', 'custom event')
+
+        if date:
+            number = self.start_local.year - int(date[:4])
             name = self._vevents[self.ref].get('x-fname', None)
-            if int(bday[4:6]) == 2 and int(bday[6:8]) == 29:
+            if int(date[4:6]) == 2 and int(date[6:8]) == 29:
                 leap = ' (29th of Feb.)'
             else:
                 leap = ''
@@ -368,21 +426,22 @@ class Event(object):
                 suffix = 'rd'
             else:
                 suffix = 'th'
-            return '{name}\'s {number}{suffix} birthday{leap}'.format(
-                name=name, number=number, suffix=suffix, leap=leap,
+            return '{name}\'s {number}{suffix} {desc}{leap}'.format(
+                name=name, number=number, suffix=suffix, desc=description, leap=leap,
             )
         else:
             return self._vevents[self.ref].get('SUMMARY', '')
 
-    def update_summary(self, summary):
+    def update_summary(self, summary: str) -> None:
         self._vevents[self.ref]['SUMMARY'] = summary
 
     @staticmethod
-    def _can_handle_alarm(alarm):
+    def _can_handle_alarm(alarm) -> bool:
         """
         Decides whether we can handle a certain alarm.
         """
-        return alarm.get('ACTION') == 'DISPLAY' and isinstance(alarm.get('TRIGGER').dt, dt.timedelta)
+        return alarm.get('ACTION') == 'DISPLAY' and \
+            isinstance(alarm.get('TRIGGER').dt, dt.timedelta)
 
     @property
     def alarms(self):
@@ -412,27 +471,67 @@ class Event(object):
         self._vevents[self.ref].subcomponents = components
 
     @property
-    def location(self):
+    def location(self) -> str:
         return self._vevents[self.ref].get('LOCATION', '')
 
-    def update_location(self, location):
+    def update_location(self, location: str) -> None:
         if location:
             self._vevents[self.ref]['LOCATION'] = location
         else:
             self._vevents[self.ref].pop('LOCATION')
 
     @property
-    def categories(self):
-        return self._vevents[self.ref].get('CATEGORIES', '')
+    def attendees(self) -> str:
+        addresses = self._vevents[self.ref].get('ATTENDEE', [])
+        if not isinstance(addresses, list):
+            addresses = [addresses, ]
+        return ", ".join([address.split(':')[-1]
+                          for address in addresses])
 
-    def update_categories(self, categories):
-        if categories.strip():
-            self._vevents[self.ref]['CATEGORIES'] = categories
+    def update_attendees(self, attendees: List[str]):
+        assert isinstance(attendees, list)
+        attendees = [a.strip().lower() for a in attendees if a != ""]
+        if len(attendees) > 0:
+            # first check for overlaps in existing attendees.
+            # Existing vCalAddress objects will be copied, non-existing
+            # vCalAddress objects will be created and appended.
+            old_attendees = self._vevents[self.ref].get('ATTENDEE', [])
+            unchanged_attendees = []
+            vCalAddresses = []
+            for attendee in attendees:
+                for old_attendee in old_attendees:
+                    old_email = old_attendee.lstrip("MAILTO:").lower()
+                    if attendee == old_email:
+                        vCalAddresses.append(old_attendee)
+                        unchanged_attendees.append(attendee)
+            for attendee in [a for a in attendees if a not in unchanged_attendees]:
+                item = icalendar.prop.vCalAddress(f'MAILTO:{attendee}')
+                item.params['ROLE'] = icalendar.prop.vText('REQ-PARTICIPANT')
+                item.params['PARTSTAT'] = icalendar.prop.vText('NEEDS-ACTION')
+                item.params['CUTYPE'] = icalendar.prop.vText('INDIVIDUAL')
+                item.params['RSVP'] = icalendar.prop.vText('TRUE')
+                # TODO use khard here to receive full information from email address
+                vCalAddresses.append(item)
+
+            self._vevents[self.ref]['ATTENDEE'] = vCalAddresses
         else:
-            self._vevents[self.ref].pop('CATEGORIES', False)
+            self._vevents[self.ref].pop('ATTENDEE')
 
     @property
-    def description(self):
+    def categories(self) -> str:
+        try:
+            return self._vevents[self.ref].get('CATEGORIES', '').to_ical().decode('utf-8')
+        except AttributeError:
+            return ''
+
+    def update_categories(self, categories: List[str]) -> None:
+        assert isinstance(categories, list)
+        self._vevents[self.ref].pop('CATEGORIES', False)
+        if categories:
+            self._vevents[self.ref].add('CATEGORIES', categories)
+
+    @property
+    def description(self) -> str:
         return self._vevents[self.ref].get('DESCRIPTION', '')
 
     def update_description(self, description):
@@ -442,19 +541,29 @@ class Event(object):
             self._vevents[self.ref].pop('DESCRIPTION')
 
     @property
-    def _recur_str(self):
+    def _recur_str(self) -> str:
         if self.recurring:
             recurstr = ' ' + self.symbol_strings['recurring']
         else:
             recurstr = ''
         return recurstr
 
-    def format(self, format_string, relative_to, env={}, colors=True):
+    @property
+    def _alarm_str(self) -> str:
+        if self.alarms:
+            alarmstr = ' ' + self.symbol_strings['alarming']
+        else:
+            alarmstr = ''
+        return alarmstr
+
+    def format(self, format_string: str, relative_to, env=None, colors: bool=True):
         """
         :param colors: determines if colors codes should be printed or not
         :type colors: bool
         """
-        attributes = dict()
+        env = env or {}
+
+        attributes = {}
         try:
             relative_to_start, relative_to_end = relative_to
         except TypeError:
@@ -572,14 +681,19 @@ class Event(object):
 
         attributes["repeat-symbol"] = self._recur_str
         attributes["repeat-pattern"] = self.recurpattern
+        attributes["alarm-symbol"] = self._alarm_str
         attributes["title"] = self.summary
+        attributes["organizer"] = self.organizer.strip()
         attributes["description"] = self.description.strip()
         attributes["description-separator"] = ""
         if attributes["description"]:
             attributes["description-separator"] = " :: "
         attributes["location"] = self.location.strip()
-        attributes["all-day"] = allday
+        attributes["attendees"] = self.attendees
+        attributes["all-day"] = str(allday)
         attributes["categories"] = self.categories
+        attributes['uid'] = self.uid
+        attributes['url'] = self.url
 
         if "calendars" in env and self.calendar in env["calendars"]:
             cal = env["calendars"][self.calendar]
@@ -587,6 +701,7 @@ class Event(object):
             attributes["calendar"] = cal.get("displayname", self.calendar)
         else:
             attributes["calendar-color"] = attributes["calendar"] = ''
+            attributes["calendar"] = self.calendar
 
         if colors:
             attributes['reset'] = style('', reset=True)
@@ -599,33 +714,38 @@ class Event(object):
             for c in ["black", "red", "green", "yellow", "blue", "magenta", "cyan", "white"]:
                 attributes[c] = attributes[c + '-bold'] = ''
 
-        attributes['status'] = self.status
+        attributes['nl'] = '\n'
+        attributes['tab'] = '\t'
+        attributes['bell'] = '\a'
+
+        attributes['status'] = self.status + ' ' if self.status else ''
         attributes['cancelled'] = 'CANCELLED ' if self.status == 'CANCELLED' else ''
         return format_string.format(**dict(attributes)) + attributes["reset"]
 
-    def duplicate(self):
-        """duplicate this event's PROTO event
-
-        :rtype: Event
-        """
+    def duplicate(self) -> 'Event':
+        """duplicate this event's PROTO event"""
         new_uid = generate_random_uid()
         vevent = self._vevents['PROTO'].copy()
         vevent['SEQUENCE'] = 0
         vevent['UID'] = icalendar.vText(new_uid)
         vevent['SUMMARY'] = icalendar.vText(vevent['SUMMARY'] + ' Copy')
-        event = self.fromVEvents([vevent])
+        event = self.fromVEvents([vevent], locale=self._locale)
         event.calendar = self.calendar
         event._locale = self._locale
         return event
 
-    def delete_instance(self, instance):
-        """delete an instance from this event"""
+    def delete_instance(self, instance: dt.datetime) -> None:
+        """delete an instance from this event
+
+        we don't check, if that instance is an instance of the recurrence rules
+        defined in the event
+        """
         assert self.recurring
         delete_instance(self._vevents['PROTO'], instance)
 
         # in case the instance we want to delete is specified as a RECURRENCE-ID
         # event, we should delete that as well
-        to_pop = list()
+        to_pop = []
         for key in self._vevents:
             if key == 'PROTO':
                 continue
@@ -638,7 +758,7 @@ class Event(object):
             self._vevents.pop(key)
 
     @property
-    def status(self):
+    def status(self) -> str:
         return self._vevents[self.ref].get('STATUS', '')
 
 
@@ -650,16 +770,16 @@ class LocalizedEvent(DatetimeEvent):
     """
     see parent
     """
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         try:
             starttz = getattr(self._vevents[self.ref]['DTSTART'].dt, 'tzinfo', None)
         except KeyError:
             msg = (
-                "Cannot understand event {} from "
-                "calendar {}, you might want to file an issue at "
+                f"Cannot understand event {kwargs.get('href')} from "
+                f"calendar {kwargs.get('calendar')}, you might want to file an issue at "
                 "https://github.com/pimutils/khal/issues"
-                .format(kwargs.get('href'), kwargs.get('calendar'))
             )
             logger.fatal(msg)
             raise FatalError(  # because in ikhal you won't see the logger's output
@@ -686,14 +806,14 @@ class LocalizedEvent(DatetimeEvent):
             self._end = endtz.localize(self._end)
 
     @property
-    def start_local(self):
+    def start_local(self) -> dt.datetime:
         """
         see parent
         """
         return self.start.astimezone(self._locale['local_timezone'])
 
     @property
-    def end_local(self):
+    def end_local(self) -> dt.datetime:
         """
         see parent
         """
@@ -703,55 +823,56 @@ class LocalizedEvent(DatetimeEvent):
 class FloatingEvent(DatetimeEvent):
     """
     """
-    allday = False
+    allday: bool = False
 
     @property
-    def start_local(self):
+    def start_local(self) -> dt.datetime:
         return self._locale['local_timezone'].localize(self.start)
 
     @property
-    def end_local(self):
+    def end_local(self) -> dt.datetime:
         return self._locale['local_timezone'].localize(self.end)
 
 
 class AllDayEvent(Event):
-    allday = True
+    allday: bool = True
 
     @property
-    def end(self):
-        end = super(AllDayEvent, self).end
+    def end(self) -> dt.datetime:
+        end = super().end
         if end == self.start:
             # https://github.com/pimutils/khal/issues/129
-            logger.warning('{} ("{}"): The event\'s end date property '
-                           'contains the same value as the start date, '
-                           'which is invalid as per RFC 5545. Khal will '
-                           'assume this is meant to be single-day event '
-                           'on {}'.format(self.href, self.summary, self.start))
+            logger.warning(f'{self.href} ("{self.summary}"): The event\'s end '
+                           'date property contains the same value as the start '
+                           'date, which is invalid as per RFC 5545. Khal will '
+                           'assume this is meant to be a single-day event on '
+                           f'{self.start}')
             end += dt.timedelta(days=1)
         return end - dt.timedelta(days=1)
 
     @property
-    def duration(self):
+    def duration(self) -> dt.timedelta:
         try:
             return self._vevents[self.ref]['DURATION'].dt
         except KeyError:
             return self.end - self.start + dt.timedelta(days=1)
 
 
-def create_timezone(tz, first_date=None, last_date=None):
+def create_timezone(
+    tz: pytz.BaseTzInfo,
+    first_date: Optional[dt.datetime]=None,
+    last_date: Optional[dt.datetime]=None
+) -> icalendar.Timezone:
     """
     create an icalendar vtimezone from a pytz.tzinfo object
 
     :param tz: the timezone
-    :type tz: pytz.tzinfo
     :param first_date: the very first datetime that needs to be included in the
     transition times, typically the DTSTART value of the (first recurring)
     event
-    :type first_date: datetime.datetime
     :param last_date: the last datetime that needs to included, typically the
     end of the (very last) event (of a recursion set)
     :returns: timezone information
-    :rtype: icalendar.Timezone()
 
     we currently have a problem here:
 
@@ -767,42 +888,42 @@ def create_timezone(tz, first_date=None, last_date=None):
     easy solution, we'd really need to ship another version of the OLSON DB.
 
     """
-    if isinstance(tz, pytz.tzinfo.StaticTzInfo):
+    if isinstance(tz, StaticTzInfo):
         return _create_timezone_static(tz)
 
     # TODO last_date = None, recurring to infinity
 
     first_date = dt.datetime.today() if not first_date else to_naive_utc(first_date)
-    last_date = dt.datetime.today() if not last_date else to_naive_utc(last_date)
+    last_date = first_date + dt.timedelta(days=1) if not last_date else to_naive_utc(last_date)
     timezone = icalendar.Timezone()
     timezone.add('TZID', tz)
 
     dst = {
         one[2]: 'DST' in two.__repr__()
-        for one, two in iter(tz._tzinfos.items())
+        for one, two in iter(tz._tzinfos.items())  # type: ignore
     }
     bst = {
         one[2]: 'BST' in two.__repr__()
-        for one, two in iter(tz._tzinfos.items())
+        for one, two in iter(tz._tzinfos.items())  # type: ignore
     }
 
     # looking for the first and last transition time we need to include
-    first_num, last_num = 0, len(tz._utc_transition_times) - 1
-    first_tt = tz._utc_transition_times[0]
-    last_tt = tz._utc_transition_times[-1]
-    for num, transtime in enumerate(tz._utc_transition_times):
-        if transtime > first_tt and transtime < first_date:
+    first_num, last_num = 0, len(tz._utc_transition_times) - 1  # type: ignore
+    first_tt = tz._utc_transition_times[0]  # type: ignore
+    last_tt = tz._utc_transition_times[-1]  # type: ignore
+    for num, transtime in enumerate(tz._utc_transition_times):  # type: ignore
+        if first_date > transtime > first_tt:
             first_num = num
             first_tt = transtime
-        if transtime < last_tt and transtime > last_date:
+        if last_tt > transtime > last_date:
             last_num = num
             last_tt = transtime
 
-    timezones = dict()
+    timezones: Dict[str, icalendar.Component] = {}
     for num in range(first_num, last_num + 1):
-        name = tz._transition_info[num][2]
+        name = tz._transition_info[num][2]  # type: ignore
         if name in timezones:
-            ttime = tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None)
+            ttime = tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None)  # type: ignore
             if 'RDATE' in timezones[name]:
                 timezones[name]['RDATE'].dts.append(
                     icalendar.prop.vDDDTypes(ttime))
@@ -815,12 +936,12 @@ def create_timezone(tz, first_date=None, last_date=None):
         else:
             subcomp = icalendar.TimezoneStandard()
 
-        subcomp.add('TZNAME', tz._transition_info[num][2])
+        subcomp.add('TZNAME', tz._transition_info[num][2])  # type: ignore
         subcomp.add(
             'DTSTART',
-            tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None))
-        subcomp.add('TZOFFSETTO', tz._transition_info[num][0])
-        subcomp.add('TZOFFSETFROM', tz._transition_info[num - 1][0])
+            tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None))  # type: ignore
+        subcomp.add('TZOFFSETTO', tz._transition_info[num][0])  # type: ignore
+        subcomp.add('TZOFFSETFROM', tz._transition_info[num - 1][0])  # type: ignore
         timezones[name] = subcomp
 
     for subcomp in timezones.values():
@@ -829,13 +950,11 @@ def create_timezone(tz, first_date=None, last_date=None):
     return timezone
 
 
-def _create_timezone_static(tz):
-    """create an icalendar vtimezone from a pytz.tzinfo.StaticTzInfo
+def _create_timezone_static(tz: StaticTzInfo) -> icalendar.Timezone:
+    """create an icalendar vtimezone from a StaticTzInfo
 
     :param tz: the timezone
-    :type tz: pytz.tzinfo.StaticTzInfo
     :returns: timezone information
-    :rtype: icalendar.Timezone()
     """
     timezone = icalendar.Timezone()
     timezone.add('TZID', tz)
@@ -843,7 +962,7 @@ def _create_timezone_static(tz):
     subcomp.add('TZNAME', tz)
     subcomp.add('DTSTART', dt.datetime(1601, 1, 1))
     subcomp.add('RDATE', dt.datetime(1601, 1, 1))
-    subcomp.add('TZOFFSETTO', tz._utcoffset)
-    subcomp.add('TZOFFSETFROM', tz._utcoffset)
+    subcomp.add('TZOFFSETTO', tz._utcoffset)  # type: ignore
+    subcomp.add('TZOFFSETFROM', tz._utcoffset)  # type: ignore
     timezone.add_component(subcomp)
     return timezone
